-- @Author: superm
-- @Date:   2017-09-23 17:03:34
-- @Last Modified by: ZYY
-- @Last Modified time: 2019-03-07 10:27:14
-- @File Name: ActivityQueueBaseModel.lua

-- region
local This , super = class(..., "Territory.TerrQueue")
local GamePlayer = require("Player.GamePlayer")
local ActivityBaseModel = require("Activity.ActivityBaseModel")
local ActivityExchangeModel = require("Activity.ActivityExchangeModel")
local ActivityRankModel = require("Activity.ActivityRankModel")
local ActivityUtil = require("Activity.ActivityUtil")

function This:create(...)
	local ret = {}
	setmetatable(ret,self)
	ret:init(...)
	return ret
end

function This:initBaseData( info )
	self.m_isDirty = true
	self.m_isInited = false
    self.m_index = info.id or 0
    self.m_config = info.config
    self.m_type = self.m_config.cmd
    self.m_upStartTime = info.start_time or 0
    self.m_upEndTime = info.end_time or 0
	self.m_allObejectState = false
	self.m_allExchangeObejectState = false
	self.m_eobjects = {}
	self.m_robjects = {}
	if self.m_index > 0 then
		self.m_uiconfig = ActivityUtil:getUIConfigList(self.m_index)
	end
	self:updateTimeInfo(info)
end

function This:updateTimeInfo( info )

	if info.start_time then
	    self.m_upStartTime = info.start_time
	    self.m_isInited = GamePlayer:getServerTime() >= self.m_upStartTime
	end
	
	if info.end_time then
	    self:setEndTime(info.end_time)
	end

	if info.start_time and info.end_time then
		local curtime = GamePlayer:getServerTime() 
		self.m_isInited = info.start_time < 0 or info.start_time < 0 or (curtime >= info.start_time and curtime <= info.end_time)
	end

	if info.start_level then
		self:setOriginalLevel(info.start_level)
	end
end

function This:updateBaseData( info )
	self:setDirty(false)
    self:updateTimeInfo( info )
    self:setCurNum(info)
end

function This:isSuitable(config, actId, orgLevel)
    if config.activity_id and actId ~= config.activity_id then
        return false
    end
    if orgLevel and orgLevel > 0 and config.level_limit then
        local startLev, endLev = unpack(config.level_limit)
        if startLev > 0 and endLev > 0 and startLev <= endLev then
            return orgLevel >= startLev and orgLevel <= endLev
        end
    end
    return true
end

function This:initBeforeUpdateState(info, createHandler)

	if info.initing then
		self.m_objects = {}
		self.m_eobjects = {}
		self.m_robjects = {}
		if self:getType() > 0 then
        	local queueId = self:getId()
			local typeName = self:getTypeName()
        	local originLevel = self:getOriginalLevel()
			local config, actConfigList
			if self:hasPickupBehaviour() and createHandler then
				config = require("Metadatas." .. typeName .. "Config")
				actConfigList = {}
				for cfgIndex, cfg in ipairs(config) do
					if self:isSuitable(cfg, queueId, originLevel) then
						local act = createHandler(queueId, cfgIndex, cfg, self)
				    	self.m_objects[act:getId()] = act
				    	table.insert(actConfigList, cfg)
					end
				end
			    local finishHandler = self.initGroupsInfo
			    if finishHandler and actConfigList then
				    finishHandler(self, actConfigList)
			    end
			end

            if self:hasExchangeBehaviour() then
				config = self:initExchangeObjList(typeName)
			    local finishHandler = self.initExchangeGroupsInfo
			    if finishHandler and config then
				    finishHandler(self, config)
			    end
			end

            if self:hasRankBehaviour() then
				self:initRankObjList(typeName)
			end
		end
	end
end

function This:getExchangeCreator()
    return nil
end

function This:initExchangeObjList( typeName )
	if self:hasExchangeBehaviour() then
		local class = self:getExchangeCreator()
		if not class then
			class = ActivityExchangeModel
		end

		local econfig = require("Metadatas." .. typeName .. "ExchangeConfig")
		local ret = {}
        local queueId = self:getId()
		for cfgIndex, cfg in ipairs(econfig) do
			if not cfg.activity_id or queueId == cfg.activity_id then
				table.insert(ret, cfg)
				local act = class:create({id = cfgIndex, config = cfg})
		    	self.m_eobjects[act:getId()] = act
				act:setQueue(self)
			end
		end
		return ret
	end
end

function This:initRankObjList( typeName )
	if self:hasRankBehaviour() then
		local class = self:getRankCreator()
		if not class then
			class = ActivityRankModel
		end

		local rconfig = require("Metadatas." .. typeName .. "RankingConfig")
		local ret = {}
        local queueId = self:getId()
		for cfgIndex, cfg in pairs(rconfig) do
			if not cfg.activity_id or queueId == cfg.activity_id then
				table.insert(ret, cfg)
				local act = class:create({id = cfgIndex, config = cfg})
		    	self.m_robjects[act:getId()] = act
				act:setQueue(self)
			end
		end
		return ret
	end
end

function This:selfUpdateInfo(info, createHandler)
	if info.reward_index then
    	for _, act in pairs(self.m_objects) do
            if act then
                act:setRewardIndex(0)
            end
    	end

		for _, actInfo in ipairs(info.reward_index) do
			local act = self.m_objects[actInfo.id]
            if act then
                act:setRewardIndex(actInfo.num)
            end
		end		
	end

	if info.reward_num then
    	for _, act in pairs(self.m_objects) do
            if act then
                act:setRewardNumber(0)
            end
    	end

		for _, actInfo in ipairs(info.reward_num) do
			local act = self.m_objects[actInfo.id]
            if act then
                act:setRewardNumber(actInfo.num)
            end
		end		
	end

    if info.reward_list then
    	for _, act in pairs(self.m_objects) do
            if act then
                act:setState(ActObjectState.kLocked)
            end
    	end

		for _, activityId in ipairs(info.reward_list) do
			local act = self.m_objects[activityId]
            if act then
                act:setCompleted()
            end
		end
	end

	if info.exchange_list then
    	for _, act in pairs(self.m_eobjects) do
            if act then
                act:setState(ActObjectState.kLocked)
            end
    	end

		for _, exchangeInfo in ipairs(info.exchange_list) do
            local act = self.m_eobjects[exchangeInfo.id]
            if act then
                act:setCompleteTimes(exchangeInfo.num)
            end
		end
	end
end

function This:endUpdate()
	self.m_state = TerrObjectState.kLocked
    super.endUpdate(self)
    self.m_isDirty  = true
end

function This:isInited()
    return self.m_isInited
end

function This:isDirty()
    return self.m_isDirty  
end

function This:isAllwaysDirty()
	local value = self:getDirtyType()
	local param = ActDirtyReasonType.kAllWays
	
	-- param = ActDirtyReasonType.kDay
	-- if bit.band(value, param) == param then
	-- 	return not self:isSameDay()
	-- end
    return bit.band(value, param) == param
end

function This:isTimeDirtyType()
	local value = self:getDirtyType()
	local param = ActDirtyReasonType.kTime
    return bit.band(value, param) == param
end

function This:isRechargeDirtyType()
	local value = self:getDirtyType()
	local param = ActDirtyReasonType.kRecharge
    return bit.band(value, param) == param
end

function This:isUIRechargeDirtyType( index )
	local value
	if self.m_uiconfig and index then
		config = self.m_uiconfig[index]
		if config then
			value = config.refresh_type
		end
	end
	if not value then
		return false
	end
    return value == ActDirtyReasonType.kRecharge 
end

function This:hasShowLimit(indice)
	if self.m_uiconfig then
		local index = indice or 1
		local config = self.m_uiconfig[index]
		if not config or not config.showlimit or config.showlimit <= 0 then
			return
		end
		return config.showlimit
	end
end

function This:isNeedBuyValid()
    return true
end

function This:needBuy()
	if self.m_upStartTime < 0 and self.m_upEndTime < 0 then
		return false
	end
	return true
end

function This:getOriginalLevel()
end

function This:setOriginalLevel(value)
end

function This:getType()
	return self.m_type
end

function This:getConfigType()
    return self.m_config.type
end

function This:getTitleType()
    return self.m_config.title_type 
end

function This:hasResTypeWith( value )
	if not self.m_config.res_type then
		return false
	end

    return bit.band(self.m_config.res_type, value) == value
end

function This:getName(index)
	local name
	if self.m_uiconfig and index then
		config = self.m_uiconfig[index]
		if config then
			name = config.name
		end
	end

	if not name then
		name = self.m_config.name 
	end
	return name
end

function This:getFreshMode()
    return self.m_config.fresh or ActFreshMode.kNormal 
end

function This:getTypeName()
    return self.m_config.type_name
end

function This:getPanelName()
	local ret = self.m_config.panel
	if ret then
		return ret
	end
    return self:getTypeName()
end

function This:getConfigPanelName()
	return self.m_config.panel
end

function This:getRedDotKey()
    return self.m_config.r_key
end

function This:getPriority()
    return self.m_config.priority
end

function This:getUITypeName()
	if self.m_uiconfig then
		local name = self.m_uiconfig.source
		if name then
			return name
		end
	end
    return "Activity.ActivityBaseUI"
end

function This:getCSFileName()
	if self.m_uiconfig then
		local name = self.m_uiconfig.csfile
		if name then
			return name
		end
	end
end

function This:getSubCSFileName()
	if self.m_uiconfig then
		local name = self.m_uiconfig.subcsfile
		if name then
			return name
		end
	end
end

function This:getConfigSubCmd()
    return self.m_config.subcmd
end

function This:getConfigOtherCmd()
    return self.m_config.othercmd or 12014
end

function This:getDirtyType()
    return self.m_config.refresh_type 
end

function This:getCurNum()
    return self.m_curNum
end

function This:setCurNum( info )
    self.m_curNum = 0
end

function This:setDirty(value, reason)
	if value and bit.band(self:getDirtyType(), reason) == reason then
    	self.m_isDirty = value
    elseif not value then
        self.m_isDirty = false
    end
end

function This:getDefaultGroup()
	return tostring(1)
end

function This:getGroupListCount()
    return self.m_groupNameListCount or 0
end

function This:getGroupNameList( config, key, handler, exchange)
	local nameList, groupList, groupRewardList
    if not exchange then
	    nameList = self.m_groupNameList
    else
	    nameList = self.m_exchangeGroupNameList
    end

    if not nameList and config then
	    if not exchange then
		    self.m_group = {}
		    self.m_groupNameList = {}
		    self.m_groupRewardList = {}

		    nameList = self.m_groupNameList
		    groupList = self.m_group
		    groupRewardList = self.m_groupRewardList
	    else
		    self.m_exchangeGroupNameList = {}
		    self.m_exchangeGroupList = {}
		    self.m_exchangeGroupRewardList = {}
		    nameList = self.m_exchangeGroupNameList
		    groupList = self.m_exchangeGroupList
		    groupRewardList = self.m_exchangeGroupRewardList
	    end
    	self.m_groupNameListCount = 0

	    for _, actConfig in ipairs(config) do
	    	local actId = actConfig.id
	    	if actConfig[key] > 0 then
	    		local value = handler(actConfig[key])
		    	if not groupList[value] then
		    		self.m_groupNameListCount = self.m_groupNameListCount + 1
					groupList[value] = {}
					table.insert(nameList, value)
		    	end
		    	table.insert(groupList[value], actId)
		    end
	    end
	end
	return nameList
end

function This:getGroupListBy(index, exchange)
	local objects, nameList, groupList
	if not exchange then
		objects = self.m_objects or self.m_eobjects
		groupList = self.m_group
		nameList = self.m_groupNameList
	else
		objects = self.m_eobjects
		groupList = self.m_exchangeGroupList
		nameList = self.m_exchangeGroupNameList
	end

	local key = nameList[index]
	local found = groupList[key]
	local ret = {}
	if found then
		for _, actId in ipairs(found) do
			table.insert(ret, objects[actId])
		end
	end
	return ret
end

function This:getFilterIdsList( index )
	local ret = {}
	if self.m_uiconfig then
		local config = self.m_uiconfig[index]
		if config then
			ret = config.filter_ids or {}
		end
	end
	return ret
end

function This:getGroupRewardListBy(index, exchange)
	local objects, nameList, groupList, groupRewardList
	if not exchange then
		objects = self.m_objects or self.m_eobjects
		nameList = self.m_groupNameList
		groupList = self.m_group
		groupRewardList = self.m_groupRewardList
	else
		objects = self.m_eobjects
		nameList = self.m_exchangeGroupNameList
		groupList = self.m_exchangeGroupList
		groupRewardList = self.m_exchangeGroupRewardList
	end

	local ret = groupRewardList[index]
	if not ret then
		ret = {}
		local key = nameList[index]
		local found = groupList[key]
		if found then
			for _, actId in ipairs(found) do
				local activity = objects[actId]
				if activity then
					local reward = activity:getRewardList()
		            for i, itemInfo in ipairs(reward) do
		            	local num = ret[itemInfo[1]] or 0
		            	ret[itemInfo[1]] = num + itemInfo[2]
		            end
				end
			end
		end
		groupRewardList[index] = ret
	end
	return ret
end

function This:getRewardList(rtype)
    for id, act in pairs(self.m_objects) do
        if not act:isCompleted() then
            return act:getRewardList(rtype)
        end
    end
end

function This:setCompleted()
    for id, act in pairs(self.m_objects) do
        if not act:isCompleted() then
            act:setCompleted()
            break
        end
    end
end

function This:requestPickUp()
    for id, act in pairs(self.m_objects) do
        if not act:isCompleted() then
            act:requestPickUp()
            break
        end
    end
end

function This:getState()
	if self:isInited() then
		return not self:isLocked()
	end
end

function This:getProgressByIndex(index)
end

function This:getObjectStateByGroupIndex( num, exchange )
    local ret = false
    local objects = self:getGroupListBy(num, exchange)
    for _, act in pairs(objects) do
        if act:isOpening() then
            ret = true
            break
        end
    end
    return ret  
end

function This:getObjectStateByRuleIndex( index )
    if not index or not self.m_uiconfig then
        return self:getAllObjectState()
    end

    local filterIdsList = self:getFilterIdsList(index)
    if not filterIdsList or #filterIdsList <= 0 then
        return self:getAllObjectState()
    end

    local ret = table.find(self.m_objects, function(id, activity)
        if not activity:isOpening() then
            return false
        end

        local fid = activity:getProgressType()
        if not fid then
            return false
        end
        return table.find(filterIdsList, function(_, v) return fid == v end)
    end)
    
    return ret 
end

function This:getAllExchangeObjects()
    return self.m_eobjects
end

function This:getAllRankObjects()
    return self.m_robjects
end

function This:isAllObjectAutoUpdate()
    return true
end

function This:hasRankBehaviour()
    return false
end

function This:getRankCreator()
end

function This:hasExchangeBehaviour()
    return false
end

function This:showExchangeBehaviour()
    return false
end

function This:hasPickupBehaviour()
    return true
end

function This:isNotStart()
    return false
end

function This:updateObjects(isInit, oldState)
	if self:isAllObjectAutoUpdate() then
		if not isInit then
			for _, act in pairs(self.m_objects) do
		        if act and act:isLocked() then
		            act:updateState()
		        end
			end 
		end
	end

	self:getAllObjectState(true)
	self:getAllExchangeObjectState(true)
end

function This:getAllObjectState(update)
	if self:hasPickupBehaviour() then
		if update then
			self.m_allObejectState = false
			for id, act in pairs(self.m_objects) do
		    	if act:isOpening() then
		    		self.m_allObejectState = true
		    		break
		    	end
			end
		end
	    return self.m_allObejectState  
	elseif self:hasExchangeBehaviour() then
		return self:getAllExchangeObjectState(update)
	end
end

function This:getAllExchangeObjectState(update)
	if self:hasExchangeBehaviour() then
		if update then
			self.m_allExchangeObejectState = false
			for id, act in pairs(self.m_eobjects) do
		    	if act:isOpening() then
		    		self.m_allExchangeObejectState = true
		    		break
		    	end
			end
		end
	    return self.m_allExchangeObejectState  
	end
end

function This:getEObjectBy(handler)
    for id, obj in pairs(self.m_eobjects) do
    	if handler and handler(obj) then
    		return obj
    	end
    end
    return nil
end

function This:showTimer()
    return false
end

function This:showCondition()
    return false
end

function This:getActivityCreateDays()
	if self:getStartTime() <= 0 then
		return 0
	end

    local startDate = os.serverdate("*t", self:getStartTime() )
    local curDate = os.serverdate("*t", GamePlayer:getServerTime() )
	return (curDate.year -startDate.year) * 365 + curDate.yday -startDate.yday +1
end

function This:markDataTimeBy( time )
    self.m_dataRefreshTime = os.serverdate("*t", time or GamePlayer:getServerTime()).yday
end

function This:isSameDay()
    local record = self.m_dataRefreshTime 
	if not record then
		return false
	end
	
    local nowTime = os.serverdate("*t", GamePlayer:getServerTime()).yday
    return nowTime == record 
end

function This:isLevelOpen()
	if self.m_config.audit and GamePlayer:isAuditVersion() then
		return false
	end
	
	local level = GamePlayer:getLevel()
    return self.m_config.open_level <= level
end

function This:setSpecialUnlocked(value)
    self.m_levelUnLocked = value
end

function This:isSpecialUnlocked()
    return self.m_levelUnLocked
end

function This:getUIDesc( index )
	local desc
	if self.m_uiconfig and index then
		config = self.m_uiconfig[index]
		if config then
			desc = config.des
		end
	end
	return desc
end

--获取配置信息region begin

--获取配置信息region end

--endregion