local This = class(...)
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local ItemUtil = require("Bag.ItemUtil")
local GamePlayer = require("Player.GamePlayer")
local EquipManager = require("Equip.EquipManager")
local FabaoManager = require("Fabao.FabaoManager")
local RecoveryConfig = require("Metadatas.RecoveryConfig")
local ShopMappingConfig = require("Metadatas.ShopMappingConfig")

local Operation = {
    kEquipReborn = 1,
    kFabaoReborn = 2,
    kFabaoSell = 3,
    kRecycle = 4,
    kLordEquipSell = 5,
    kRuneSell = 6,--符石
    kMinWen = 7,
    kTacticsSell = 8, --战法
}

function This:init()
    self.m_data = {
        items = {},
    }
    self.m_observers = {}
    self.m_aloneNums = {}
    self:initListener()
end

function This:createData(message)
    local data = {}
    local items = {}
    for _, item in ipairs(message.bag_item_list) do
        local itemModel
        local eq = EquipManager:getEquip(item.id)
        if not eq then
            itemModel = ItemUtil.createItemModel(item)
        end
        itemModel = itemModel or eq:getItem()
        items[itemModel:getUid()] = itemModel
    end
    data.items = items
    return data
end

function This:initListener()
	EventDispatcher:addListener(self, "equip_init",self.onGetBagData)
	EventDispatcher:addListener(self, "equip_changed",self.onGetBagDataChange)
    EventDispatcher:addListener(self, 10602, self.onSellItemResp)
    EventDispatcher:addListener(self, 10603, self.onUseItemResp)
    EventDispatcher:addListener(self, 10604, self.onSynthesizeSpallResp)
    EventDispatcher:addListener(self, 10605, self.onRecieveWarmingReply)
    EventDispatcher:addListener(self, 10442, self.onSellItemNewReply)
    EventDispatcher:addListener(self, 10606, self.onSellItemNewReply)
    EventDispatcher:addListener(self, 10607, self.onReceiveUniversalStoreItemsReply)
    EventDispatcher:addListener(self, 10608, self.onReceiveUniversalStoreItemInfoReply)
end

function This:onRecieveWarmingRequest()
    Socket:sendAndLock(10605)
end

function This:addObserver(observer, callbacks)
    self.m_observers[observer] = callbacks or observer
end

function This:removeObserver(observer)
    self.m_observers[observer] = nil
end

function This:callObservers(handlerName, ...)
    for observer, callbacks in pairs(self.m_observers) do
        local callback = callbacks[handlerName]
        if callback then
           callback(observer, ...)
        end
    end
end

function This:getData()
    return self.m_data
end

function This:loadData()
    Socket:sendAndLock(10600, {})
end

function This:getItem(id)
    for k,item in pairs(self.m_data.items) do
        if item.id == id then
            return item
        end
    end 
end

function This:getItemByUid(uid)
    return self.m_data.items[uid]
end

function This:getItemList(id)
	local result = {}
    for k,item in pairs(self.m_data.items) do
        if item.id == id then
			table.insert(result,item)
        end
    end
	return result
end

function This:getItemCount(id)
    local item = self:getItem(id)
    if item then
        return item:getCount()
    else
        return 0
    end
end

function This:getItemListByType(typeID, sortComp)
    local itemList = {}
    for k,item in pairs(self.m_data.items) do
        if ItemUtil.getType(item.id) == typeID then
            table.insert(itemList, item)
        end
    end
    if sortComp and #itemList > 1 then
        table.sort(itemList, sortComp)
    end
    return itemList
end

function This:getItemListByTypeList(typeIDList, sortComp)
    local itemList = {}
    
    for k,item in pairs(self.m_data.items) do
        local itype = ItemUtil.getType(item.id)
        if table.contain(typeIDList, itype) and item and item:isVisible() then
            table.insert(itemList, item)
        end
    end

    if sortComp and #itemList > 1 then
        table.sort(itemList, sortComp)
    end
    return itemList
end

local function idsListSortComp(a, b)
    local item1, item2 = a.item, b.item
    if item1.id ~= item2.id then
        local quality1, quality2 = item1:getQuality(), item2:getQuality()
        if quality1 ~= quality2 then
            return quality1 < quality2
        end
        return item1.id < item2.id
    else
        return a.index > b.index
    end
end

local function commonSelectSortComp(item1, item2)
    if item1.id ~= item2.id then
        local quality1, quality2 = item1:getQuality(), item2:getQuality()
        if quality1 ~= quality2 then
            return quality1 < quality2
        end

        return item1.id < item2.id
    else
        return item1._id > item2._id
    end
end


function This:getItemListByFilter(filterHandler, sortComp)
    local itemList = {}
    local count, itemCount = 0, 0
    for k,item in pairs(self.m_data.items) do
        local fresult, fcount = filterHandler(item)
        if fresult then
            local itemId = item:getId()
            local iCount = item:getCount() - fcount
            if iCount > 0 then
                table.insert(itemList, item)
                count = count + 1
                itemCount = itemCount + iCount
            end
        end
    end
    if count > 1 then
        table.sort(itemList, sortComp or commonSelectSortComp)
    end
    return itemList, count, itemCount
end

function This:getItemIdsListByFilter(filterHandler, sortComp)
    local itemList = {}, {}
    local count = 0
    for k,item in pairs(self.m_data.items) do
        local fresult, fcount = filterHandler(item)
        if fresult then
            local itemId = item:getId()
            local iCount = item:getCount() - fcount
            if iCount > 0 then
                for i=1, iCount do
                    table.insert(itemList, {index = i, item = item})
                end
            count = count + iCount
            end
        end
    end

    if count > 1 then
        table.sort(itemList, sortComp or idsListSortComp)
    end
    return itemList, count, count
end

function This:getHeroSpallCount(heroId)
    return self:getItemCount(heroId + 10000)
end

local function commonSortComp(item1, item2)
    local state1, state2 = item1:canUsed(), item2:canUsed()
    if state1 ~= state2 then
        return state1
    end    

    local quality1, quality2 = item1:getQuality(), item2:getQuality()
    if quality1 ~= quality2 then
        return quality1 > quality2
    end

    if item1.id ~= item2.id then
        return item1.id < item2.id
    else
        return item1._id < item2._id
    end
end

local function spallSortComp(item1, item2)
    local state1 = item1:canUsed()
    local state2 = item2:canUsed()
    if state1 ~= state2 then
        return state1
    end

    local ptype1, ptype2 = item1:getProductType(), item2:getProductType()
    if ptype1 ~= ptype2 then
        return ptype1 > ptype2
    end

    local quality1, quality2 = item1:getQuality(), item2:getQuality()
    if quality1 ~= quality2 then
        return quality1 > quality2
    end

    if item1.id ~= item2.id then
        return item1.id < item2.id
    else
        return item1._id < item2._id
    end
end

function This:getPropsItemList()
    return self:getItemListByTypeList({ItemCommonType.kSkin,ItemCommonType.kProps, ItemCommonType.kPropPack,ItemCommonType.kSpecialItem,ItemCommonType.kWarfare}, commonSortComp)
end

function This:getPropsItemListForInScription()
    return self:getItemListByTypeList({ItemCommonType.kInscription},commonSortComp)
end

function This:getSpallItemList()
    return self:getItemListByType(ItemCommonType.kSpall, spallSortComp)
end

function This:getAnimalSpallItemList()

    local itemList = {}
    local spallList =  self:getSpallItemList()
    for k,item in pairs(spallList) do
        if ItemUtil.isAnimalSpall(item.id) then
            table.insert(itemList, item)
        end
    end
    --for k,item in pairs(self.m_data.items) do
    --    if ItemUtil.isAnimalSpall(item.id)then
    --        table.insert(itemList, item)
    --    end
    --end
    --
    if #itemList > 1 then
        table.sort(itemList,function(a,b)
            if a:getQuality() ~= b:getQuality() then
                return a:getQuality() > b:getQuality()
            else
                return a.id > b.id
            end
        end)
    end
    return itemList
end
-------------------------------------------------------

function This:onGetBagData(message)
    self.m_data = self:createData(message)
    self:updateAloneCount()
    self:callObservers("loadData")
    -- local recruitItemCount = self:getItemCount(116)
    -- EventDispatcher:addMsg("common_reddot_multi", {recruitItem1 = recruitItemCount >= 1, recruitItem5 = recruitItemCount >= 5})
end

function This:onGetBagDataChange(message)
    local items = self.m_data.items
    local itemList = {}
    local addItemList ={}
    local isZMLChange = false
    local isHasAlone = false
    local function showGoods(goodsList)
        if not os.isWin32 or #goodsList == 0 then return end
        local goodsStr = ""
        local colorId = "A2"
        for i , goods in ipairs(goodsList) do
            local name = ItemUtil.getName(goods[1])
            local num = goods[2]
            local str = ""
            if num > 0 then
                colorId = "A2"
                str = string.format("+%d %s\n",num,name)
                goodsStr = goodsStr .. str
            elseif num < 0 then
                colorId = "R0"
                str = string.format("%d %s\n",num,name)
                goodsStr = goodsStr .. str
            end
        end

        if goodsStr ~= "" then -- 数量没有发生变化的物品，不需要提示
            g_scene:showMoneyTips(goodsStr, { delayFactor = 1.5, colorId = colorId } )
        end
    end
    local goodsList = {}
    for i, pitem in ipairs(message.bag_item_list) do
        local curItem = items[pitem.id]
        local diffNum = pitem.num - (curItem and curItem:getCount() or 0)
        if (not curItem or pitem.type_id == curItem:getId()) and diffNum > 0 then
            local item = {id = pitem.type_id, count = diffNum}
            table.insert(itemList, item)
        end
        local id = 0
        if curItem ~= nil then
            id = curItem:getId()
            if pitem.num <= 0 then
                print("删除道具", pitem.id)
                id = curItem:getId()
                curItem = nil
                items[pitem.id] = nil
            else
                curItem:updateInfo(pitem)
                id = curItem:getId()
            end
        else
            local item
            local eq = EquipManager:getEquip(pitem.id)
            if eq then item = eq:getItem() end
            item = item or ItemUtil.createItemModel(pitem)
            items[item:getUid()] = item
            print("增加道具", item:getId())
            id = item:getId()
            table.insert(addItemList,item)
        end
        table.insert(goodsList,{id,diffNum})
		if pitem.type_id == 116 then
			isZMLChange = true
		end

        if NeedAloneCountType[ItemUtil.getType(pitem.type_id)] then
            isHasAlone = true
        end
    end
    showGoods(goodsList)
    if isHasAlone then
        self:updateAloneCount()
    end
    self:callObservers("itemUpdate", itemList)
    if #addItemList>0 then
        EventDispatcher:dispatcher("equip_changed_add",addItemList)
    end
    if isZMLChange then
        local recruitItemCount = self:getItemCount(116)
        EventDispatcher:dispatcher("item_zml_changed",recruitItemCount)
    end
end

function This:onSellItemResp(message)
    if message.err_code == 0 then
        local money_type = message.money_type
        local money_num = message.money_num
        local itemList = {}
        for i, mtype in ipairs(money_type) do
            local itemID = ItemUtil.getItemIdByMoneyType(mtype)
            local item = {id=itemID, count=money_num[i]}
            table.insert(itemList, item)
        end
        self:callObservers("sellItem", itemList)
    else
        g_scene:handleErrorCode(message.err_code)
    end
end

function This:onUseItemResp(message)
    if message.err_code == 0 then
        local reward_list = {}
        local keyList = {{"get_item_num", "get_item_id"}, {"resource_num", "resource_type"}}

        for _, keyPair in ipairs(keyList) do
            local numList = message[keyPair[1]]
            local typeList = message[keyPair[2]]
            for i, num in ipairs(numList) do
                if num > 0 then
                    table.insert(reward_list, {id=typeList[i], num=num})
                end
            end
        end
        self:callObservers("useItem", reward_list)
    else
        g_scene:handleErrorCode(message.err_code)
    end
end

function This:onSynthesizeSpallResp(message)
    if message.err_code == 0 then
        self:callObservers("synthesizeSpall", message.get_item)
    else
        g_scene:handleErrorCode(message.err_code)
    end
end
----------------------------------

function This:sellItem(item, count)
    local itype = ItemUtil.getType(item.id)
    if itype == ItemCommonType.kProps or itype == ItemCommonType.kSpall or itype ==ItemCommonType.kSkin  then
        if not item or item:getCount() < count then
            g_scene:handleErrorCode("invalidAction")
            return
        end
    elseif itype == ItemCommonType.kEquip then
        local equip = EquipManager:getEquip(item._id)
        if not equip or equip:getItem():getCount() < count then
            g_scene:handleErrorCode("invalidAction")
            return
        end
    elseif itype == ItemCommonType.kFaBao then
        local fabao = FabaoManager:getFabao(item._id)
        if not fabao or fabao:asItem():getCount() < count then
            g_scene:handleErrorCode("invalidAction")
            return
        end
    else
        g_scene:handleErrorCode("invalidAction")
        return
    end
    Socket:sendAndLock(10602, {id=item._id, num=count})
end

function This:useItem(item, count, selectedId)
    if not count or count < 1 then
        count = 1
    end
    
    local itype = ItemUtil.getType(item.id or item:getId())
    if itype ~= ItemCommonType.kSkin and itype ~= ItemCommonType.kProps and itype ~= ItemCommonType.kPropPack and itype ~= ItemCommonType.kSpecialItem and itype ~= ItemCommonType.kHammer and itype ~= ItemCommonType.kWarfare then
        g_scene:handleErrorCode("invalidAction")
        return
    end

    if not item or count <= 0 or count > item:getCount() then
        g_scene:handleErrorCode("invalidAction")
        return
    end
    Socket:sendAndLock(10603, {id=item._id, num=count, type_id = selectedId or 0})
end

function This:synthesizeSpall(item, count)
    if not count or count < 1 then
        count = 1
    end

    local itype = ItemUtil.getType(item.id)
    if itype ~= ItemCommonType.kSpall then
        g_scene:handleErrorCode("invalidAction")
        return
    end

    if not item then
        g_scene:handleErrorCode("invalidAction")
        return
    end

    if item:isHeroSpall() and require("Hero.HeroManager"):isGotHero(item:getTargetId()) then
        g_scene:handleErrorCode("invalidAction")
        return
    end

    local useNum = item:getSynthesizeCost() * count
    if useNum <= 0 or useNum > item:getCount() then
        g_scene:handleErrorCode("spallLack")
        return
    end
    Socket:sendAndLock(10604, { id = 0, target_id = item:getId(), use_num = useNum } )
end

function This:exchangeItem( id, targetId )
    Socket:sendAndLock(10604, {id = id, taiget_id = targetId})
end

function This:onRecieveWarmingReply( msg )
    if msg.err_code == 0 then
        self:callObservers("onRecieveWarming")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onSellItemNewRequest(itemlist, operation)
    if not itemlist or type(itemlist) ~= "table" then
        return
    end

    local _, testItem = next(itemlist)
    if not testItem then return end

    if not operation then
        if type(testItem) == "number" then
            for index, info in ipairs(itemlist) do
                itemlist[index] = {id = info, num = 1}
            end
            testItem = itemlist[1]
        end
        local paramList = {item_id = {}, item_count = {}}
        if not testItem.id then
            for _, info in ipairs(itemlist) do
                local id, num = unpack(info)
                table.insert(paramList.item_id, id)
                table.insert(paramList.item_count, num)
            end
        else
            for _, info in ipairs(itemlist) do
                table.insert(paramList.item_id, info.id)
                table.insert(paramList.item_count, info.num)
            end
        end 
        Socket:send(10606, paramList)
    else
        local operate = function (uid, op)
            local item = self:getItemByUid(uid)
            if item then
                local operationType = Operation.kEquipReborn
                if op == ItemOperation.kReborn then
                    if not item:canReborn() then
                        return
                    elseif item:isFabao() then
                        operationType = Operation.kFabaoReborn
                    end
                elseif item:isFabao() then
                    operationType = Operation.kFabaoSell
                elseif item:isLordEquip() then
                    operationType = Operation.kLordEquipSell
                elseif item:isInscription() then
                    operationType = Operation.kMinWen
                elseif item:isRune() then
                    operationType = Operation.kRuneSell
                elseif item:checkCommonType(ItemCommonType.kTactics) then
                    operationType = Operation.kTacticsSell
                end
                print_tmp({item_id = uid, type = operationType})
                Socket:send(10442, {item_id = uid, type = operationType})
            end
        end
                
        if type(testItem) == "number" then
            for _, uid in ipairs(itemlist) do
                operate(uid, operation)
            end
        else
            for _, info in ipairs(itemlist) do
                local uid = info.id or info[1]
                operate(uid, operation)
            end
        end
    end
end

function This:getRecycleCost( target, recycleType )
    recycleType = recycleType or RecycleType.kReborn
    local id = 100 * recycleType + (target or RecycleTargetType.kHero)

    local curIndex = 0
    if recycleType == RecycleType.kReborn then
        if target == RecycleTargetType.kHero then
            local HeroManager = require("Hero.HeroManager")
            if HeroManager:isHeroResetFree() then
                return 0
            end
            curIndex = HeroManager:getHeroResetTimes() + 1
        elseif target == RecycleTargetType.kEquip then
            curIndex = (self.m_equipRebornNum or 0) + 1
        else
            curIndex = (self.m_fabaoRebornNum or 0) + 1
        end
    end

    if not self.m_recyclePriceList then
        self.m_recyclePriceList = {}
    end
    local priceList = self.m_recyclePriceList[id]
    if not priceList then
        priceList = {}
        local config = RecoveryConfig[id]
        local maxTimes = 0
        for index, times in ipairs(config.times) do
            table.insert(priceList, {
                times = times,
                price = config.price[index]
            })
            maxTimes = math.max(maxTimes, times)
        end
        table.sort(priceList, function ( priceA, priceB )
            return priceA.times <  priceB.times
        end)
        priceList.maxTimes = maxTimes
        self.m_recyclePriceList[id] = priceList
    end
    
    local found
    if priceList.maxTimes > 0 then
        if curIndex > priceList.maxTimes then
            curIndex = priceList.maxTimes
        end
    else
        return 0
    end
    for _, info in ipairs(priceList) do
        local times, price = info.times, info.price
        if not found and times >= priceList.maxTimes then
            found = price
        end
        print(curIndex ,times)
        if curIndex == times then
            found = price
            break
        end
    end
    return found
end

function This:onSellItemNewReply( msg )
    if msg.err_code == 0 then
        local operation = msg.type
        if operation and operation ~= Operation.kFabaoSell then
            if operation ~= Operation.kFabaoReborn then
                self.m_equipRebornNum = msg.equip_reborn_num
            end
            
            if operation ~= Operation.kEquipReborn then
                self.m_fabaoRebornNum  = msg.jewelry_reborn_num
            end
        else
            operation = ItemOperation.kSell
        end
        if operation == Operation.kRuneSell then
             g_scene:showReward(msg.reward_list)
        end
        self:callObservers("onSellItemNew", operation)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:isItemOrResEnough( id, num )
    if ItemUtil.getType(id) == ItemCommonType.kGameRes then
        local switch = {
            [101] = function(num)
                return GamePlayer:getGold() >= num, "goldLack"
            end,
            [102] = function(num)
                return GamePlayer:getDiamond() >= num, "diamondLack"
            end,
            [107] = function(num)
                return GamePlayer:getZhangong() >= num
            end,
            [110] = function(num)
                return GamePlayer:getSoul() >= num
            end,
            [125] = function()
                return GamePlayer:getJingjibi() >= num
            end,
            [127] = function()
                return GamePlayer:getContributionbi() >= num
            end,
            [178] = function()
                return GamePlayer:getMonumentCoin() >= num
            end,
        }
        return switch[id](num)
    else
        return self:getItemCount(id) >= num
    end
end

function This:getItemOrResCount(id, itemInfo)
    local itype = ItemUtil.getType(id)
    if itype == ItemCommonType.kGameRes then
        local switch = {
            [101] = function()
                return GamePlayer:getGold() 
            end,
            [102] = function()
                return GamePlayer:getDiamond() 
            end,
            [107] = function(num)
                return GamePlayer:getZhangong()
            end,
            [110] = function(num)
                return GamePlayer:getSoul()
            end,
            [125] = function()
                return GamePlayer:getJingjibi()
            end,
            [127] = function()
                return GamePlayer:getContributionbi()
            end,
            [161] = function()
                return GamePlayer:getPeakContestbi()
            end,
            [168] = function()
                return GamePlayer:getSkillReel()
            end,
            [176] = function()
                return GamePlayer:getJunGong()
            end,
            [252] = function()
                -- local LegionMapManager = require("Map.LegionMapManager")
                -- return LegionMapManager:getOre()
            end,
            [302] = function()
                return GamePlayer:getAnimalSoul()
            end,
            [307] = function()
                return GamePlayer:getStone()
            end,
            [335] = function()
                return GamePlayer:getRefinedScore()
            end,
            [178] = function()
                return GamePlayer:getMonumentCoin()
            end,
        }
        return switch[id](num)

    elseif NeedAloneCountType[itype] then
        return self.m_aloneNums[id] or 0

    elseif itype ~= ItemCommonType.kEquip then
        local item = self:getItem(id)
        if item then
            if itemInfo then
                itemInfo.itemUid = item:getUid()
                itemInfo.itemId = id
            end
            return item:getCount()
        end
        return 0
    end    
end

function This:getItemOrResEnought(items,showTip)
    if items then
        for _,item1 in pairs(items) do
            local id ,needNum = item1[1],item1[2]
            local curentNum = self:getItemOrResCount(id)
            if needNum>curentNum then
                if showTip then
                    g_scene:showTips(Serialize[60714] .. ItemUtil.getName(id) .. Serialize[60715])
                end
                return false
            end
        end
        return true
    end
    return false
end

-- 通用商店物品购买
function This:onBuyItemInUniversalStoreRequest(storeSystemType, goodsIndex)
    Socket:sendAndLock(10607,{buy_type=storeSystemType, buy_index=goodsIndex})
end
function This:onReceiveUniversalStoreItemsReply(msg)
    if msg.err_code ~= 0 then
        g_scene:handleErrorCode(msg.err_code)
        return
    end
    self:callObservers("onReceiveUniversalStorePurchaseResult", msg)
end

-- 通用商店物品信息
function This:onGetUniversalStoreItemInfoRequest(storeSystemType)
    Socket:sendAndLock(10608,{buy_type=storeSystemType})
end
function This:onReceiveUniversalStoreItemInfoReply(msg)
    if msg.err_code ~= 0 then
        g_scene:handleErrorCode(msg.err_code)
        return
    end
    self:callObservers("onReceiveUniversalStoreItemInfo", msg)
end

-- 统计不叠加的物品数量
function This:updateAloneCount()
    local nums = {}
    for k, item in pairs(self.m_data.items) do
        local itype = item:getType()
        if NeedAloneCountType[itype] then
            if item:getEquipCardId() == 0 then
                local id = item:getId()
                nums[id] = (nums[id] or 0) + 1
            end
        end
    end
    self.m_aloneNums = nums
end