-- @Author: superm
-- @Date:   2018-06-28 17:03:38
-- @Last Modified by:   zheng
-- @Last Modified time: 2018-09-12 11:43:23
-- @File Name: FamilyBuildManager.lua
-- @Description: 

local This , super = class(..., "Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local FamilyBuilding = require("Family.FamilyBuilding")
local Signals = require("SignalSystem.Signals")
local SignalSystem = require("SignalSystem.SignalSystem")

function This:init()
	super.init(self)
    self:initListener()
    self:cleanAllBuilding()
end

function This:initListener()
	EventDispatcher:addListener(self, 12119, self.onGetFamilyBuildInfoReply)
    EventDispatcher:addListener(self, 12120, self.onUpgradeFamilyBuildReply)
	EventDispatcher:addListener(self, 12121, self.onUpgradeFamilyBuildReply)
    EventDispatcher:addListener(self, 12131, self.onChangeStateReply)
end

function This:removeListener()
    EventDispatcher:removeListener(self,12119)
    EventDispatcher:removeListener(self,12120)
    EventDispatcher:removeListener(self,12121)
    EventDispatcher:removeListener(self,12131)
end

function This:cleanAllBuilding()
    self.m_buildingList = {}
    for _, btype in pairs(FamilyBuildType) do
        self.m_buildingList[btype] = FamilyBuilding:createByBuildType(btype)
    end
end

function This:getBuildByType(buildType)
    return self.m_buildingList[buildType]
end

function This:getBuildingLevel(buildType)
    local build = self.m_buildingList[buildType]
    if build then
        return build:getLevel()
    end
    return 0
end

function This:getDeductionSum()
    local sum = 0
    for _, build in ipairs(self.m_buildingList) do
        if build:getLevel() > 0 and not build:isNormal() then
            sum = sum + build:getDeductionCost()
        end
    end 
    return sum
end

-- 获取军团特权相关数值
function This:getPrerogativeValue(buildType, field)
    local build = self:getBuildByType(buildType)
    if not build or build:getLevel() <= 0 then
        return 0
    end

    local ret = 0
    local config = build:getConfig()
    local data
    if build:isNormal() then
        data = config.routine_prerogative
    else
        data = config.maintain_prerogative
    end

    if data then
        for _, info in ipairs(data) do
            if info[1] == field then
                ret = ret + info[2]
            end
        end
    end
    return ret
end

-- 商店开放槽位数目
function This:getStoreSlotNum(storeType, buildType)
    local field = FamilyPrerogativeType.kStoreContrbSlotNum
    if storeType == StoreType.kFamilyFunds then
        field = FamilyPrerogativeType.kStoreFundsSlotNum
    end
    return self:getPrerogativeValue(buildType or FamilyBuildType.kShangDian, field)
end

-- 捐献加成
function This:getFamilyDonateAddition(buildType)
    local field = FamilyPrerogativeType.kFamilyDonateAddition
    return self:getPrerogativeValue(buildType or FamilyBuildType.kShangHui, field)
end

-- 分红品质
function This:getFamilyShareQuality(buildType)
    local field = FamilyPrerogativeType.kFundsDividend
    return self:getPrerogativeValue(buildType or FamilyBuildType.kYiShiTing, field)
end

-- 人数上限加成
function This:getFamilyPeopleAddition(buildType)
    local field = FamilyPrerogativeType.kPeopleNum
    return self:getPrerogativeValue(buildType or FamilyBuildType.kYiShiTing, field)
end

-- 建筑是否解锁
function This:isBuildUnlock(buildType)
    return self:getBuildingLevel(buildType) > 0
end

-- 按状态获取建筑
function This:getBuildingByState(state)
    local buildList = {}
    for _, build in ipairs(self.m_buildingList) do
        if build:getState() == state then
            table.insert(buildList, build)
        end
    end
    return buildList
end

function This:isInNormalState()
    for _, build in ipairs(self.m_buildingList) do
        if build:getLevel() > 0 and not build:isNormal() then
            print(build:getType() .. " maintain")
            return false
        end
    end
    print("all buildings are normal")
    return true
end

function This:onGetFamilyBuildInfoRequest(silence)
    if not silence then
        Socket:sendAndLock(12119, {})
    else
        Socket:send(12119, {})
    end
end

function This:onGetFamilyBuildInfoReply( msg )
    if msg.err_code == 0 then
        for _, buildInfo in ipairs(msg.building_list) do
            local build = self:updateBuilding( buildInfo )
            print(build:getName() .. ":Lv." .. build:getLevel())
        end
        self:notifyObservers("onGetFamilyBuildInfo")
        SignalSystem.distributeSignal(Signals.FamilyPanelBuildTabRedDot)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:updateBuilding( info )
    local building = self.m_buildingList[info.type]  
    if not building then
        building = FamilyBuilding:create(info)
        self.m_buildingList[info.type] = building
    else
        building:updateInfo(info)
    end
    return building
end

function This:onUpgradeFamilyBuildRequest(btype, level)
    if not level or level <= 0 then
	   Socket:sendAndLock(12121, {type= btype})
    else
       Socket:sendAndLock(12120, {type= btype})
    end
end

function This:onUpgradeFamilyBuildReply( msg )
    if msg.err_code == 0 then
        for _, buildInfo in ipairs(msg.building_list) do
            local build = self:updateBuilding( buildInfo )
            print(build:getName() .. ":Lv." .. build:getLevel())
        end
        self:notifyObservers("onUpgradeFamilyBuild")
		EventDispatcher:dispatcher("family_build_change")
        SignalSystem.distributeSignal(Signals.FamilyPanelBuildTabRedDot)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
    
function This:onChangeStateRequest(btype)
    Socket:sendAndLock(12131,{type= btype})
end

function This:onChangeStateReply( msg )
    if msg.err_code == 0 then
        for _, buildInfo in ipairs(msg.building_list) do
            local build = self:updateBuilding( buildInfo )
            print(build:getName() .. ":Lv." .. build:getLevel())
        end
        self:notifyObservers("onChangeState")
        EventDispatcher:dispatcher("family_build_change")
        SignalSystem.distributeSignal(Signals.FamilyPanelBuildTabRedDot)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end