

local This , super = class(...,"Base.BaseModel")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local FamilyManager = require("Family.FamilyManager")
local HeroNpcConfig = require("Metadatas.HeroNpcConfig")
local LegionBossRankConfig = require("Metadatas.LegionBossRankConfig")
local PersonalBossRankConfig = require("Metadatas.PersonalBossRankConfig")
local HeroManager = require("Hero.HeroManager")
local FamilyBossManager = require("FamilyBoss.FamilyBossManager")

local ActiveWeek = {
	[1] = true,
	[2] = true,
	[3] = true,
	[4] = true,
	[5] = true,
	[6] = true,
	[7] = true,
}

function This:init(info)
	super.init(self)

	self.m_id = info.id or 0
	self.m_config = info.config
	self:initData()
end

function This:initData()
	self.m_isOpen = false
	self.m_isLingQu = false

	self.m_roleRankInfo = {}
	self.m_roleRankReward = {}
	self.m_roleRankList = {}

	self.m_legionRankInfo = {}
	self.m_legionRankReward = {}
	self.m_legionRankList = {}

	self.m_challengeNum = 0
	self.m_challengeReward = {}
	self.m_challengeHeros = {}
	
	self.m_escape = 0
	self.m_lastPhase = self:getPhase()
end

function This:updateInfo(msg)
	self.m_isOpen = (msg.boss_state == 1)
	self.m_isLingQu = (msg.reward_times > 0)
	self:setChallengeNum(msg.challenge_times or 0)
	self:setChallengeHeros(msg.challenge_card_list or {})
	self:setMyRoleRankInfo(msg.role_rank or {})
	self:setMyLegionRankInfo(msg.group_rank or {})
end

function This:updateRank(msg)
	local rankType = msg.type --1个人排名；2-军团排名	

	local list = {}
	for i, v in ipairs(msg.rank_list) do
		local t = {}
		t.id = v.id
		t.name = v.name
		t.damage = v.damage
		t.people = v.participator_count
		t.rank = v.rank
		t.rankType = rankType
		table.insert(list, t)
	end

	if #list >= 2 then
		table.sort(list, function (a, b)
			return a.rank < b.rank
		end)
	end

	if msg.type == 1 then 
		self.m_roleRankList = list
	elseif msg.type == 2 then
		self.m_legionRankList = list
	end

	return {type = rankType, list = list}
end

function This:getConfigVal(key, default)
	if self.m_config and self.m_config[key] ~= nil then
		return self.m_config[key]
	end
	return default
end

function This:getId()
	return self.m_id
end

-- 副本名
function This:getName()
	return self:getConfigVal("name", "")
end

-- 副本描述图
function This:getDescImage()
	return self:getConfigVal("descimg")
end

-- boss名
function This:getBossName()
	return self:getConfigVal("bossname", "")
end

-- boss描述
function This:getBossDesc()
	return self:getConfigVal("bossdesc", "")
end

-- boss图
function This:getBossImage()
	return self:getConfigVal("lordimg")
end

-- 开启资金
function This:getOpenFunds()
	return self:getConfigVal("funds", 0)
end

-- 今天的星期
function This:getTodayWeek()
	local curTime = GamePlayer:getServerTime()
	local date = os.serverdate("*t", curTime)
	return (date.wday + 5) % 7 + 1
end

-- 是否开启了副本
function This:isOpen()
	return self.m_isOpen
end

-- 今天是否有开启
function This:isActive()
	return ActiveWeek[self:getTodayWeek()]
end

-- 开启的星期文字
function This:getOpenWeekStr()
	return Serialize[61025]
end

-- 开启的时间文字
function This:getOpenTimeStr()
	local _, openHour, openMin = GameUtil:getFormatDtime( self:getConfigVal("open_time", 0) )
	local _, endHour, endMin = GameUtil:getFormatDtime( self:getConfigVal("end_time", 0) )
	return string.format("%02d:%02d~%02d:%02d", openHour, openMin, endHour, endMin)
end

-- 阶段
function This:getPhase()
	local ret = {
		type = FamilyBossPhaseType.kNotStart,
		startTime = 0,
		endTime = 0
	}

	if not self:isActive() then
		return ret
	end

	local curTime = GamePlayer:getServerTime()
	local date = os.serverdate("*t", curTime)
	local zeroTime = os.time({year = date.year, month = date.month, day = date.day, hour = 0, min = 0, sec = 0})
	local enterStart = zeroTime + self:getConfigVal("open_time", 0)
	local enterEnd = zeroTime + self:getConfigVal("end_time", 0)
	local rewardEnd = zeroTime + self:getConfigVal("reward_end_time", 0)

	if curTime < enterStart then
		ret.type = FamilyBossPhaseType.kPreview
		ret.startTime = zeroTime
		ret.endTime = enterStart - 1
		return ret
	end

	if curTime >= enterStart and curTime <= enterEnd then
		ret.type = FamilyBossPhaseType.kEnter
		ret.startTime = enterStart
		ret.endTime = enterEnd
		return ret
	end

	if curTime >= enterEnd + 1 and curTime <= rewardEnd then
		ret.type = FamilyBossPhaseType.kReward
		ret.startTime = enterEnd + 1
		ret.endTime = rewardEnd
		return ret
	end

	return ret
end

-- 检查阶段变化
function This:checkPhase()
	local newPhase = self:getPhase()
	local oldPhase = self.m_lastPhase

	if newPhase.type ~= oldPhase.type then
		self.m_lastPhase = newPhase
		FamilyBossManager:onPhaseChange({
			instance = self,
			newPhase = newPhase,
			oldPhase = oldPhase
		})
	end
end

-- 剩余的进入时间
function This:getRemainEnterTime()
	local phase = self:getPhase()

	if phase.type ~= FamilyBossPhaseType.kEnter then
		return 0
	end

	local curTime = GamePlayer:getServerTime()
	local remain = phase.endTime - curTime
	if remain < 0 then
		remain = 0
	end

	return remain
end

-- 预告时间
function This:isPreviewTime()
	local phase = self:getPhase()
	return phase.type == FamilyBossPhaseType.kPreview
end

-- 进入时间
function This:isEnterTime()
	local phase = self:getPhase()
	return phase.type == FamilyBossPhaseType.kEnter
end

-- 奖励时间
function This:isRewardTime()
	local phase = self:getPhase()
	return phase.type == FamilyBossPhaseType.kReward	
end

-- 是否有权限开启
function This:isHasPowerOpen()
	local family = FamilyManager:getMyFamily()
	return family:getSelfMemberInfo():hasPower(FamilyPower.kOpenStage)
end

-- 副本总血量
function This:getMaxHp()
	local hp = 0
	for i, v in ipairs( self:getConfigVal("squad", {}) ) do
		local id = v[1]
		hp = hp + (HeroNpcConfig[id] and HeroNpcConfig[id].kMaxHp or 0)
	end
	return hp
end

-- 是否已领取
function This:isLingQu()
	return self.m_isLingQu
end

function This:setLingQu(isLingQu)
	self.m_isLingQu = isLingQu
end

function This:getRewardList()
	local dict = {}
	local addListToDict = function (list)
		for i, v in ipairs(list) do
			local id, num = v[1], v[2]
			dict[id] = (dict[id] or 0) + num
		end
	end

	addListToDict(self.m_roleRankReward)
	addListToDict(self.m_legionRankReward)
	addListToDict(self.m_challengeReward)

	local list = {}
	for id, num in pairs(dict) do
		table.insert(list, {id,num})
	end

	return list
end

-- 是否有奖励
function This:isExistReward()
	return #self.m_roleRankReward > 0 or #self.m_legionRankReward > 0 or #self.m_challengeReward > 0
end

-- 设置个人排名信息
function This:setMyRoleRankInfo(info)
	local t = {}
	t.id = info.id or 0
	t.name = GamePlayer:getRoleName()
	t.damage = info.damage or ""
	t.rank = info.rank or 0
	t.rankType = FamilyBossRankType.kMember

	self.m_roleRankInfo = t
	self.m_roleRankReward = {}

	local rank = info.rank or 0
	for id = 1, 100 do
		local config = PersonalBossRankConfig[id]
		if not config then
			break
		end
		if rank >= config.min_rank and rank <= config.max_rank then
			self.m_roleRankReward = config.reward
			break
		elseif rank >= config.min_rank and config.max_rank == -1 then
			self.m_roleRankReward = config.reward
			break
		end
	end
end

-- 获取个人排名信息
function This:getMyRoleRankInfo()
	return self.m_roleRankInfo
end

-- 获取个人排名
function This:getRoleRank()
	return self.m_roleRankInfo.rank or 0
end

-- 获取个人排名奖励
function This:getRoleRankReward()
	return self.m_roleRankReward
end

-- 获取个人排名列表
function This:getRoleRankList()
	return self.m_roleRankList
end

-- 设置军团排名信息
function This:setMyLegionRankInfo(info)
	local t = {}
	t.id = info.id or 0
	t.name = FamilyManager:getMyFamilyName()
	t.damage = info.damage or ""
	t.people = info.participator_count or 0
	t.rank = info.rank or 0
	t.rankType = FamilyBossRankType.kFamily

	self.m_legionRankInfo = t
	self.m_legionRankReward = {}

	local rank = info.rank or 0
	for id = 1, 100 do
		local config = LegionBossRankConfig[id]
		if not config then
			break
		end
		if rank >= config.min_rank and rank <= config.max_rank then
			self.m_legionRankReward = config.reward
			break
		elseif rank >= config.min_rank and config.max_rank == -1 then
			self.m_legionRankReward = config.reward
			break
		end
	end	
end

-- 获取个人排名信息
function This:getMyLegionRankInfo()
	return self.m_legionRankInfo
end

-- 获取军团排名
function This:getLegionRank()
	return self.m_legionRankInfo.rank or 0
end

-- 获取军团排名奖励
function This:getLegionRankReward()
	return self.m_legionRankReward
end

-- 获取军团排名列表
function This:getLegionRankList()
	return self.m_legionRankList
end

-- 设置挑战次数
function This:setChallengeNum(num)
	self.m_challengeNum = num
	self.m_challengeReward = {}

	if num > 0 then
		for i, v in ipairs( self:getConfigVal("reward", {}) ) do
			self.m_challengeReward[i] = {v[1], v[2]*num}
		end
	end
end

-- 获取挑战次数
function This:getChallengeNum()
	return self.m_challengeNum
end

-- 获取最大挑战数
function This:getMaxChallengeNum()
	return self:getConfigVal("challenge_maxtime", 0)
end

-- 获取剩余挑战次数
function This:getRemainChallengeNum()
	return self:getMaxChallengeNum() - self:getChallengeNum()
end

-- 获取挑战奖励
function This:getChallengeReward()
	return self.m_challengeReward
end

-- 设置挑战过的武将
function This:setChallengeHeros(heroList)
	self.m_challengeHeros = {}
	self:addChallengeHeros(heroList)
end

-- 附加挑战过的武将
function This:addChallengeHeros(heroList)
	for i, uid in ipairs(heroList) do
		self.m_challengeHeros[uid] = true
	end	
end

-- 是否能挑战
function This:isCanChallenge(uid)
	return not self.m_challengeHeros[uid]
end

-- 能布阵的武将列表
function This:getCanEmbattleHeroList()
	local heroList = HeroManager:getAllHeroGotWithSort(function(hero)
		if not self:isCanChallenge(hero:getUid()) then
			return false
		end
		return true
	end)
	return heroList
end

function This:update(dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape >= 1 then
		self.m_escape = 0
		self:checkPhase()
	end
end

