

local This , super = class(...,"Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local LegionBossInstanceConfig = require("Metadatas.LegionBossInstanceConfig")
local GamePlayer = require("Player.GamePlayer")
local BattleDataManager = require("Battle.BattleDataManager")
local Signals = require("SignalSystem.Signals")
local SignalSystem = require("SignalSystem.SignalSystem")
local PopDialogManager = require("UI.PopDialogManager")


FamilyBossPhaseType = {
	kNotStart = 0,
	kPreview = 1,
	kEnter = 2,
	kReward = 3,
}

FamilyBossRankType = {
	kMember = 1,
	kFamily = 2,
}

function This:init()
	super.init(self)

	self.m_chapter = nil
	self.m_tipState = false
	self:initChapter()
	self:initListener()
end

function This:initListener()
	EventDispatcher:addListener(self, "day_passed_withoutnet", self.onDayPassNotify)
	EventDispatcher:addListener(self, 12117, self.onFamilyBossInfoReply)
	EventDispatcher:addListener(self, 12114, self.onFamilyBossOpenReply)
	EventDispatcher:addListener(self, 12115, self.onFamilyBossChallengeReply)
	EventDispatcher:addListener(self, 12116, self.onFamilyBossRankReply)
	EventDispatcher:addListener(self, 12159, self.onFamilyBossRewardReply)

	EventDispatcher:addListener(self, 12109, self.onFamilyChangeNotify)
	EventDispatcher:addListener(self, 12133, self.onFamilyChangeNotify)
end

function This:initChapter()
	local FamilyBossChapter = require("FamilyBoss.FamilyBossChapter")
	local config = LegionBossInstanceConfig[1]
	self.m_chapter = FamilyBossChapter:create({
		id = 1,
		config = config
	})
end

function This:getChapter()
	return self.m_chapter
end

function This:startUpdate()
	cc.Application:addToUpdateList(self)
end

function This:endUpdate()
	cc.Application:removeFromUpdateList(self)
end

function This:update(dt)
	self.m_chapter:update(dt)
end

-- 是否有军团
function This:hasFamily()
	return require("Family.FamilyManager"):hasFamily()
end

-- boss信息
function This:onFamilyBossInfoRequest()
	--print("==== onFamilyBossInfoRequest")
	if self:hasFamily() then
		Socket:sendAndLock(12117, {chapter_id = self.m_chapter:getId()})
	else
		self.m_chapter:initData()
		self:checkValid()
		self:endUpdate()
		self:notifyChange()
	end
end

function This:onFamilyBossInfoReply(msg)
	-- print("==== onFamilyBossInfoReply")
	-- print_tmp(msg)
	if msg.err_code == 0 then
		self.m_chapter:updateInfo(msg)
		self:checkValid()
		self:startUpdate()
		self:notifyChange()
	else
		g_scene:handleErrorCode(msg.err_code)
	end	
end

-- 开启boss
function This:onFamilyBossOpenRequest(chapter_id)
	Socket:sendAndLock(12114, {chapter_id = chapter_id})
end

function This:onFamilyBossOpenReply(msg)
	-- print("==== onFamilyBossOpenReply")
	-- print_tmp(msg)	
	if msg.err_code == 0 then
		require("Family.FamilyManager"):onGetMyFamilyInfoRequest()
		self:onFamilyBossInfoRequest()
	else
		g_scene:handleErrorCode(msg.err_code)
	end	
end

-- 挑战boss
function This:onFamilyBossChallengeRequest(card_id_list)
	Socket:sendAndLock(12115, {card_id_list = card_id_list})
end

function This:onFamilyBossChallengeReply(msg)
	-- print("==== onFamilyBossChallengeReply")
	-- print_tmp(msg)	
	if msg.err_code == 0 then
		self.m_chapter:setChallengeNum(self.m_chapter:getChallengeNum()+1)
		self.m_chapter:addChallengeHeros(msg.card_id_list)
		self.m_battleInfo = msg
		cc.SceneManager:switchTo("FamilyBoss.FamilyBossLoadingScene",nil,{
			msg = msg
		})
		BattleDataManager:addObserver(self)
	else
		g_scene:handleErrorCode(msg.err_code)
	end		
end

-- 查看boss排名
function This:onFamilyBossRankRequest(type)
	Socket:sendAndLock(12116, {type = type}) --1个人排名；2-军团排名
end

function This:onFamilyBossRankReply(msg)
	print("==== onFamilyBossRankReply")
	print_tmp(msg)	
	if msg.err_code == 0 then
		local ret = self.m_chapter:updateRank(msg)
		self:notifyObservers("onFamilyBossRankReply", ret)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

-- 领取奖励
function This:onFamilyBossRewardRequest(chapter_id)
	Socket:sendAndLock(12159, {chapter_id = chapterId})
end

function This:onFamilyBossRewardReply(msg)
	print("==== onFamilyBossRewardReply")
	print_tmp(msg)		
    if msg.err_code == 0 then
    	self.m_chapter:setLingQu(true)
    	g_scene:showReward( self.m_chapter:getRewardList() )
        self:notifyChange()
        require("Family.FamilyManager"):onGetMyFamilyInfoRequest()
    else
        g_scene:handleErrorCode(msg.err_code)
    end	
end

-- 检查有效性（军团不存在 或者 副本未激活 时关闭所有相关界面）
function This:checkValid()
	if not self:hasFamily() or not self.m_chapter:isActive() then
		local ui = {
			["FamilyBossPanel"] = true,
			["FamilyBossRankPanel"] = true,
			["FamilyBossRewardPreviewPanel"] = true,
			["FamilyBossSelectHeroPanel"] = true,
			["FamilyBossEmbattlePanel"] = true,
			["FamilyBossHelpPanel"] = true,
			["FamilyBossMyRewardPanel"] = true,
			["FamilyBossComePanel"] = true,
		}
		for _, dialog in pairs(g_scene:getAllDialogs()) do
			if ui[dialog._NAME] then
				print("----- family boss close ui " .. dialog._NAME)
				dialog:close()
				self:removeObserver(dialog)
			end
		end
		return false
	end
	return true
end

-- 军团改变
function This:onFamilyChangeNotify(msg)
	cc.Application:getTimer():setTimer(0.1, function ()
		self:onFamilyBossInfoRequest()
	end, "family_boss_check")
end

-- 新的一天
function This:onDayPassNotify()
	self.m_chapter:initData()
	self:checkValid()
	self:notifyChange()
end

-- 阶段改变
function This:onPhaseChange(info)
	self:checkValid()
	self:notifyChange()
	self:onFamilyBossInfoRequest()
end

-- 红点
function This:isRedDot()
	local chapter = self:getChapter()
	if self:hasFamily() and chapter:isActive() then
		if chapter:isEnterTime() then
			return chapter:getRemainChallengeNum() > 0
		end
		if chapter:isRewardTime() then
			return chapter:isExistReward() and chapter:getChallengeNum() > 0 and not chapter:isLingQu()
		end
	end
	return false
end

-- 通知改变
function This:notifyChange()
	self:notifyObservers("onFamilyBossChange")
	SignalSystem.distributeSignal(Signals.FamilyPanelAffairsTabRedDot)
end

---------------------------------------------------
function This:onBattleVictory()
	BattleDataManager:removeObserver(self)
	g_scene:openDialog("FamilyBoss.FamilyBossOverPanel",self.m_battleInfo)
end

function This:onBattleFailed()
	BattleDataManager:removeObserver(self)
	g_scene:openDialog("FamilyBoss.FamilyBossOverPanel",self.m_battleInfo)
end

function This:onBattleSkip()
	BattleDataManager:removeObserver(self)
	g_scene:openDialog("FamilyBoss.FamilyBossOverPanel",self.m_battleInfo)
	-- local chapter = self:getChapter()
	-- if chapter:isActive() then
	-- 	PopDialogManager:addPopDialog({scene = "MenuScene", name = "FamilyBoss.FamilyBossPanel", arg = {}, level = DialogLevel.kForLevel})
	-- end
	-- cc.SceneManager:switchTo("Home.MenuScene")
end

function This:getTipState()
	return self.m_tipState
end

function This:setTipState(yes)
	self.m_tipState = yes
end