

local This = class(...)
local HeroManager = require("Hero.HeroManager")
local HeroTacticsSlot = require("HeroTactics.HeroTacticsSlot")
local HeroTacticsSkill = require("HeroTactics.HeroTacticsSkill")
local HeroTacticsSuit = require("HeroTactics.HeroTacticsSuit")
local HeroTacticsRefine = require("HeroTactics.HeroTacticsRefine")
local ArgConfig = require("Metadatas.ArgConfig")
local DepotManager = require("Depot.DepotManager")
local GameUtil = require("Util.GameUtil")


function This:create(hero)
	local o = {}
	setmetatable(o,self)
	o:init(hero)
	return o
end

function This:init(hero)
	self.m_hero = hero
	self.m_heroId = self.m_hero:getId()
	self.m_slots = {}
	self.m_skill = nil
	self.m_suit = nil
	self.m_refine = nil

	self:initSlot()
	self:initSkill()
	self:initSuit()
	self:initRefine()
end

-- 槽位
function This:initSlot()
	self.m_slots = {}

	for id=1, 5 do
		local slot = HeroTacticsSlot:create({id = id, hero = self.m_hero})
		self.m_slots[id] = slot
	end
end

function This:getSlot(id)
	return self.m_slots[id]
end

function This:getAllSlots()
	return self.m_slots
end

function This:isExistEmptySlot()
	local isEmptySlot = false
	for i, slot in ipairs(self.m_slots) do
	    if not slot:isWear() then
	        isEmptySlot = true
	        break
	    end
	end
	return isEmptySlot
end

function This:getSlotItemUidList()
	local list = {}
	for i, slot in ipairs(self.m_slots) do
		list[i] = slot:getItemUid()
	end	
	return list
end

-- 技能
function This:initSkill()
	self.m_skill = HeroTacticsSkill:create({hero = self.m_hero, slots = self.m_slots})
end

function This:getSkill()
	return self.m_skill
end

-- 套装
function This:initSuit()
	self.m_suit = HeroTacticsSuit:create({hero = self.m_hero, slots = self.m_slots})
end

function This:getSuit()
	return self.m_suit
end

-- 精练
function This:initRefine()
	self.m_refine = HeroTacticsRefine:create({hero = self.m_hero, slots = self.m_slots})
end

function This:getRefine()
	return self.m_refine
end

-- 开启的武将等级
function This:getOpenHeroLv()
	return ArgConfig.tactics_openlevel.arg[1]
end

function This:isTacticsFuncOpen()
	local lv = self.m_hero:getLevel()
	return lv >= self:getOpenHeroLv()
end

function This:getTacticsLockTips()
	return string.format(Serialize[61452], self:getOpenHeroLv())
end

-- 是否有战法系统（蓝紫橙红品质武将才有）
function This:isHasTacticsSystem()
	return self.m_hero:getQuality() >= Quality.kBlue
end

function This:mergeAttr(mainTb, subTb)
	for k, v in pairs(subTb) do
		if not mainTb[k] then
			mainTb[k] = 0
		end
		mainTb[k] = mainTb[k] + v
	end
	return mainTb
end

-- 装备属性
function This:getTotalItemAttr()
	local ret = {}
	for k, slot in pairs(self.m_slots) do
		self:mergeAttr(ret, slot:getTotalAttr())
	end
	return ret
end

-- 技能属性
function This:getSkillAttr()
	local skill = self:getSkill()
	if not skill:isActivate() then
		return {}
	end
	return skill:getAttr()
end

-- 主面板的总属性
function This:getMainPanelAttr()
	local attrs = {}
	self:mergeAttr(attrs, self:getTotalItemAttr())
	self:mergeAttr(attrs, self:getSkillAttr())
	return attrs
end

function This:updateInfo(info)
	info = info or {}
	-- print_tmp(info)

	local idList = info.tactics_id_list or {}
	local lvList = info.tactics_pos_leve_list or {}
	for i, slot in ipairs(self.m_slots) do
		slot:setItemUid(idList[i])
		slot:setPosLv(lvList[i])
	end
	self:getRefine():updateSlots(self.m_slots)
	self:getSkill():updateSlots(self.m_slots)
	self:getSuit():updateSlots(self.m_slots)
end

function This:updateWear(pos, itemUid)
	-- print("updateWear", pos, GameUtil:int64ToString(itemUid))
	local slot = self.m_slots[pos]
	slot:setItemUid(itemUid)
	self:getSkill():updateSlots(self.m_slots)
	self:getSuit():updateSlots(self.m_slots)
end

function This:updateOneKeyWear(idList)
	-- print("------ updateOneKeyWear")
	-- print(self:uidListToString(idList))

	for i, slot in ipairs(self.m_slots) do
		slot:setItemUid(idList[i])
	end
	self:getSkill():updateSlots(self.m_slots)
	self:getSuit():updateSlots(self.m_slots)
end

function This:updateAllPosLv(lvList)
	for i, slot in ipairs(self.m_slots) do
		slot:setPosLv(lvList[i])
	end
	self:getRefine():updateSlots(self.m_slots)
	self:getSkill():updateSlots(self.m_slots)
	self:getSuit():updateSlots(self.m_slots)	
end

function This:updateStarUp(itemUid)
	for i, slot in ipairs(self.m_slots) do
		if slot:getItemUid() == itemUid then
			slot:setItemUid(itemUid)
			break
		end
	end
	self:getSkill():updateSlots(self.m_slots)
	self:getSuit():updateSlots(self.m_slots)	
end

function This:getTotalAttr()
	local ret = {}

	self:mergeAttr(ret, self:getTotalItemAttr())
	self:mergeAttr(ret, self:getSkillAttr())
	self:mergeAttr(ret, self:getSuit():getAddSuitAttr())
	
	return ret
end

function This:getRebornReturn(cost)
	cost = cost or {}

	for i, slot in ipairs(self.m_slots) do
		-- 穿的战法
		local item = slot:getItem()
		if item then
			local id = item:getId()
			cost[id] = (cost[id] or 0) + 1
		end

		-- 强化的消耗
		for id, num in pairs(slot:getAllStrengthenCost()) do
			cost[id] = (cost[id] or 0) + num
		end
	end

	return cost
end

function This:getBestWearList()
	-- 背包里最好的
	local best = {}
	for k, item in pairs(DepotManager:getData().items) do
		if item:checkCommonType(ItemCommonType.kTactics) and item:getEquipCardId() == 0 then
	        local tacType = item:getTacticsType()
		    if not best[tacType] then
		    	best[tacType] = item
		    else
		    	best[tacType] = item:compareTactics(best[tacType])
		    end
	    end
	end	

	-- 先替换已穿最好的
	local list = {}
	local isChange = false
	for i, slot in ipairs(self.m_slots) do
	    local item = slot:getItem()
	    if item then
	    	local tacType = item:getTacticsType()
	    	list[i] = item:compareTactics(best[tacType]):getUid()
	    	best[tacType] = nil
	    else
	    	list[i] = 0
	    end
	end

	-- 剩下能穿的
	local remainList = {}
	for k, item in pairs(best) do
		table.insert(remainList, item)
	end
	if #remainList > 1 then
		table.sort(remainList, function (a, b)
			return a:getTacticsType() < b:getTacticsType()
		end)
	end

	-- 最终穿戴列表
	for i, slot in ipairs(self.m_slots) do
		local oldUid = slot:getItemUid()
		if slot:isUnlock() and not slot:isWear() then
			if #remainList > 0 then
				local item = table.remove(remainList, 1)
				list[i] = item:getUid()
			end
		end
		if oldUid ~= list[i] then
			isChange = true
		end
	end

	-- print("one1", self:uidListToString(self:getSlotItemUidList()))
	-- print("one2", self:uidListToString(list))

	return list, isChange
end

-- 一键穿戴红点
function This:isOneKeyWearRedDot()
	local list, isChange = self:getBestWearList()
	return isChange
end

-- 入口红点
function This:isRedDot()
	if not self:isHasTacticsSystem() or not self:isTacticsFuncOpen() then
		return false
	end

	if self:getRefine():isRedDot() then
		return true
	end

	for i, slot in ipairs(self.m_slots) do
		if slot:isUpgradeStarRedDot() then
			return true
		end
	end

	if self:isOneKeyWearRedDot() then
		return true
	end

	return false
end

function This:uidListToString(uidList)
	local text = "{"
	for i, id in ipairs(uidList) do
		if id > 0 then
			text = text .. GameUtil:int64ToString(id)
		else
			text = text .. tostring(id)
		end
		if i < #uidList then
			text = text .. ", "
		end
	end
	text = text .. "}"
	return text
end