local This, super = class(..., "Dialog")
local GamePlayer = require("Player.GamePlayer")
local MailManager = require("Mail.MailManager") 
local FamilyUtil = require("Family.FamilyUtil")
local UpdateLayer = require("UI.UpdateLayer")
local MailItemView = require("Mail.MailItemView")

function This:initData()
    self.m_playerName = GamePlayer:getRoleName()
    self.m_selectMailID = 0
end

function This:initNode()
	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/EmailMain.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

	self.scrollView = self.m_csNode.scrollView
	self.m_csNode.itemlist:setDirection("horizontal")
	self.m_csNode.itemlist.m_direction = "right"

    self.m_listLayer = UpdateLayer:create({
        itemWidth = 449,
        itemHeight = 103,
        column = 1,
        spaceX =0,
        spaceY =5,
        scrollView = self.scrollView
    })

    local mailList = MailManager:getMailList()
    local createFunc = function (data, i)
        return self:createMailItemView(data)
    end
    local finishFunc = function ()
        self:updateMailSelect()
        self:runListAppearAction()
    end
    self.m_listLayer:setDataList(mailList, createFunc, finishFunc)
    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setAnchorPoint(0.5, 0)
    local w, h = self.m_listLayer:getContentSize()
    self.scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)

    if #mailList == 0 then
        self:updateMailSelect()
    end
end

function This:initListener()
    MailManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    MailManager:removeObserver(self)
end

function This:initTouchEvent()
    self.m_csNode.clearButton:setTouchEnded(function()
		if MailManager:hasCanClearMail() then
			MailManager:clearMailList()
		else
			g_scene:showTips(Serialize["mail_clear_tips"])
		end
    end)

    self.m_csNode.receiveButton:setTouchEnded(function()
        MailManager:receiveMail(self.m_selectMailID)
    end)

    self.m_csNode.deleteButton:setTouchEnded(function()
        MailManager:deleteMail(self.m_selectMailID)
    end)

	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)

    self.m_csNode.allreceive:setTouchEnded(function()
        MailManager:onReceiveAllMailRequest()
    end)
end

function This:updateMailList()
    local mailList = MailManager:getMailList()

    if #mailList == 0 then
        self.m_listLayer:clearNode(true, true)
        self:updateMailSelect()
    else
        local createFunc = function (data, i)
            return self:createMailItemView(data)
        end

        local finishFunc = function ()
            self:updateMailSelect()
        end

        self.m_listLayer:setDataList(mailList, createFunc, finishFunc)
    end
end

function This:updateMailSelect()
    local views = self.m_listLayer:getAllViews()
    local mid = 0

    for i, node in ipairs(views) do
        if node:isVisible() then
            if mid == 0 then
                mid = node.mail.id
            end
            if self.m_selectMailID == node.mail.id then
                mid = self.m_selectMailID
                break
            end
        end 
    end

    self:onSelectMail(mid)
end

function This:onSelectMail(mid)
    if self.m_selectMailID ~= mid then
        self.m_selectMailID = mid
    end

    self:updateClearButton()
    self:updateSelectNode()
    self:updateContent()
end

function This:updateSelectNode()
    local views = self.m_listLayer:getAllViews()
    for _, node in pairs(views) do
        node:setSelectState(node.mail.id == self.m_selectMailID)
    end
end

function This:updateContent()
    local mail = MailManager:getMail(self.m_selectMailID)
    local isVisible = mail and (self.m_selectMailID > 0) or false 
    
    self.m_csNode.bg2:setVisible(isVisible)
    self.m_csNode.content:setVisible(isVisible)
    self.m_csNode.tips:setVisible(isVisible)
    self.m_csNode.itemlist:setVisible(isVisible)
    self.m_csNode.receiveButton:setVisible(isVisible)
    self.m_csNode.deleteButton:setVisible(isVisible)
    self.m_csNode.allreceive:setVisible(isVisible)

    if not isVisible then
        return
    end

    self.m_csNode.tips:setString(Serialize["mail_honorific"] .. self.m_playerName)
    if mail.ctype ~= MailContentType.kFamily then
        self.m_csNode.content:setString(mail.content, 510)
    else
        local textNumber = string.utf8Len(mail.fromName)
        if textNumber >= 7 then
            textNumber = 76
        else
            textNumber = 76 - (7 - textNumber)
        end 
        local content = string.format("%s\n\r%78s\n%".. textNumber .. "s", mail.content, FamilyUtil:getTitleDesc(mail.familyTitle), mail.fromName)
        self.m_csNode.content:setString(content, 510)
    end
    self.m_csNode.itemlist:removeAllItems()
    if mail.itemList and #mail.itemList > 0 and not mail.isReceive then
        local items = {}
        for _, itemInfo in ipairs(mail.itemList) do
            local id = itemInfo.id
            local num = itemInfo.num
            if items[id] then
                items[id] = items[id]+num
            else
                items[id] = num
            end
        end

        self.m_csNode.deleteButton:setVisible(false)
        local width = 71
        local x = 0
		local scale = 0.6
        for id, num in pairs(items) do
            local itemNode = require("UI.ItemNode"):create(id, num)
			itemNode:setAnchorPoint(0,0.5)
            itemNode:setScale(scale)
			local w , h = itemNode:getContentSize()
			w , h = w * scale , h * scale
			self.m_csNode.itemlist:pushBackCustomItem(itemNode,w,h,0)
        end
    else
        self.m_csNode.itemlist:setVisible(false)
        self.m_csNode.receiveButton:setVisible(false)
    end
    MailManager:onReadMailRequest(self.m_selectMailID)
end

function This:updateClearButton()
    self.m_csNode.clearButton:setVisible(self.m_selectMailID > 0)
end

function This:receiveMail(data)
    local itemList = data.itemList
    if itemList and #itemList > 0 then
        g_scene:showReward(itemList)
    end

    self:updateMailList()
    self:updateContent()
end

function This:deleteMail( mailId )
    if self.m_selectMailID == mailId then
        self.m_selectMailID = 0
    end
    self:updateMailList()
    g_scene:showTips(Serialize["mail_delete_success"])
end

function This:clearMailList()
    self.m_selectMailID = 0
    self:updateMailList()
    g_scene:showTips(Serialize["mail_delete_success"])
end

function This:onNewMail()
    self:updateMailList()
end

function This:onReadMail( mail )
    self:refreshMailItemViewById(mail.id)
    self:updateContent()
end

function This:createMailItemView(mail)
    local view = MailItemView:create(mail)

    view:setTouch({
        isContainer = true,
        onEnded = function ()
            self:onSelectMail(view.mail.id)
        end,
    })

    view:setRefreshCb(function (view)
        view:setSelectState(view.mail.id == self.m_selectMailID)
    end)

    return view
end

function This:getMailItemViewById(id)
    local views = self.m_listLayer:getAllViews()
    local node = nil

    if id and id > 0 then
        for _, v in pairs(views) do
            if v.mail.id == id then
                node = v
                break
            end
        end
    end

    return node
end

function This:refreshMailItemViewById(id)
    local node = self:getMailItemViewById(id)
    local mail = MailManager:getMail(id)

    if not node or not mail then
        return
    end
    
    node:setModel(mail)
end

function This:runListAppearAction()
    local views = self.m_listLayer:getAllViews()

    for i = 1, #views do
        local node = views[i]
        if node and node:isVisible() then
            node:runActionByID("ListAppear", i*0.04)
        end
    end
end