
local This, super = class(...,"Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")

MailContentType = {
	kCommon = 0,
	kFamily = 1,
}

function This:init()
	super.init(self)
    self.m_data = {}
	self.m_data.mails = {}
	self.m_mailUnReadList = {} 
	self.m_mailUnReceiveList = {}
	self.m_mailCanDeleteList = {}
    self:initListener()
end

function This:initListener()
	EventDispatcher:addListener(self,11001,self.onInitMailInfoNotify)
	EventDispatcher:addListener(self,11002,self.onReadMailReply)
	EventDispatcher:addListener(self,11003,self.onDeleteMailReply)
	EventDispatcher:addListener(self,11004,self.onRecieveMailAttachReply)
	EventDispatcher:addListener(self,11005,self.onSendMailReply)
	EventDispatcher:addListener(self,11006,self.onGetNewMailNotify)
	EventDispatcher:addListener(self,11007,self.onClearMailOnceReply)
	EventDispatcher:addListener(self,11008,self.onReceiveAllMailReply)
end

function This:getData()
    return self.m_data
end

function This:getMail(id)
    return self.m_data.mails[id]
end

function This:getMailList()
    local mailList = {}
    local currentTime = os.time()
    for id,mail in pairs(self.m_data.mails) do
        if currentTime - mail.startTime < mail.validTime then
            table.insert(mailList, mail)
        end
    end
    if #mailList > 0 then
        table.sort(mailList, function (mail1, mail2)
        	return mail1.startTime > mail2.startTime
        end)
    end
    return mailList
end

function This:hasNewMailOrUnReceive()
	if next(self.m_mailUnReceiveList) ~= nil or next(self.m_mailUnReadList) ~= nil then
		return true
	else
		return false
	end
end

function This:hasCanClearMail()
	return next(self.m_mailCanDeleteList) ~= nil
end

function This:createMail(msgmail)
	local t = {id = msgmail.id, 
		title = msgmail.title, 
		content = msgmail.content, 
		itemList = msgmail.item_list, 
		startTime = msgmail.time, 
		validTime = msgmail.time_out,
		isRead = msgmail.is_read,
		isReceive = msgmail.is_reward,
		playerId = msgmail.id,
		fromName = msgmail.from_name,
		ctype = msgmail.type,
		familyTitle = msgmail.member_title,
	}
	return t
end

function This:onInitMailInfoRequest()
    Socket:sendAndLock(11001, {})
end

function This:onInitMailInfoNotify(msg)
	self.m_data.mails = {}
	self.m_mailUnReadList = {}
	self.m_mailUnReceiveList = {}
	self.m_mailCanDeleteList = {}
	for i , info in ipairs(msg.mail_list) do
		local mail = self:createMail(info)
		self.m_data.mails[info.id] = mail
		local isRead, hasAttach  = mail.isRead, self:mailHasAttachment(mail)
		if not isRead then
			self.m_mailUnReadList[info.id] = mail
		end

		if hasAttach then
			self.m_mailUnReceiveList[info.id] = mail
		end

		if isRead and not hasAttach then
			self.m_mailCanDeleteList[info.id] = mail
		end
	end
	EventDispatcher:addMsg("on_menu_visible_change", {name="email", value = (self.m_hasMailUnReceive or table.count(self.m_mailUnReadList) > 0)})
end

function This:onGetNewMailNotify(msg)
	local mail = self:createMail(msg.mail)
	self.m_data.mails[mail.id] = mail
	if not mail.isRead then
		self.m_mailUnReadList[mail.id] = mail
	end

	if self:mailHasAttachment(mail) then
		self.m_mailUnReceiveList[mail.id] = mail
	end
	EventDispatcher:addMsg("on_menu_visible_change", {name="email", value = true})
end

function This:onReadMailRequest(id)
	local mail = self.m_data.mails[id]
	if mail and not mail.isRead then
		Socket:sendAndLock(11002,{id = id})
	end
end

function This:onReadMailReply(msg)
	if msg.err_code == 0 then
		local mail = self:getMail(msg.mail.id)
		mail.isRead = true
		if self.m_mailUnReadList[mail.id] then
			if not self:mailHasAttachment(mail) then
				self.m_mailCanDeleteList[mail.id] = mail
			end
			self.m_mailUnReadList[mail.id] = nil
		end
		if not self:hasNewMailOrUnReceive() then
			EventDispatcher:addMsg("on_menu_visible_change", {name="email", value = false})
		end
		self:notifyObservers("onReadMail", msg.mail)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:mailHasAttachment(mail)
    return mail and mail.itemList and (#mail.itemList > 0) and (not mail.isReceive)
end

function This:receiveMail(id)
    local mail = self:getMail(id)
    if not mail then
        return
    end
    if not self:mailHasAttachment(mail) then
        g_scene:handleErrorCode("invalidAction")
        return
    end
    Socket:sendAndLock(11004, {id=id})
end

function This:onRecieveMailAttachReply(msg)
	if msg.err_code == 0 then
		local mail = self:getMail(msg.mail.id)
		if mail then
			mail.isReceive = true
			if self.m_mailUnReceiveList[mail.id] then
				self.m_mailUnReceiveList[mail.id] = nil
				self.m_mailCanDeleteList[mail.id] = mail
			end

			if table.count(self.m_mailUnReceiveList) <= 0  and table.count(self.m_mailUnReadList) <= 0 then
				EventDispatcher:addMsg("on_menu_visible_change", {name="email", value = false})
			end
			
			self:notifyObservers("receiveMail",{itemList = msg.mail.item_list})
		end
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:deleteMail(id)
    local mail = self:getMail(id)
    if not mail then
        return
    end
    if self:mailHasAttachment(mail) then
        g_scene:handleErrorCode("invalidAction")
        return
    end
	Socket:sendAndLock(11003,{id = id})
end

function This:onDeleteMailReply(msg)
	if msg.err_code == 0 then
		local mail = self:getMail(msg.id)
		if mail then
			self.m_mailCanDeleteList[mail.id] = nil
			self.m_data.mails[msg.id] = nil
			self:notifyObservers("deleteMail",msg.id)
		end
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:clearMailList()
    Socket:sendAndLock(11007,{})
end

function This:onClearMailOnceReply(msg)
	if msg.err_code ~= 0 then
		g_scene:handleErrorCode(msg.err_code)
	else
		self:notifyObservers("clearMailList")
	end
end

-- 一键领取
function This:onReceiveAllMailRequest()
	local isExistReward = false

	for id, mail in pairs(self.m_data.mails) do
		if self:mailHasAttachment(mail) then
			isExistReward = true
			break
		end
	end

	if not isExistReward then
		g_scene:showTips(Serialize[61655])
		return
	end
	
	Socket:sendAndLock(11008, {})
end

function This:onReceiveAllMailReply(msg)
	if msg.err_code == 0 then
		local itemList = {}

		-- 领的邮件
		for _, id in ipairs(msg.mail_list) do
			local mail = self:getMail(id)
			if mail then
				mail.isReceive = true
				if self.m_mailUnReceiveList[mail.id] then
					self.m_mailUnReceiveList[mail.id] = nil
					self.m_mailCanDeleteList[mail.id] = mail
				end

				for i, v in ipairs(mail.itemList) do
					table.insert(itemList, v)
				end
			end
		end

		-- 全部已读
		for id, mail in pairs(self.m_data.mails) do
			mail.isRead = true
			if self.m_mailUnReadList[mail.id] then
				if not self:mailHasAttachment(mail) then
					self.m_mailCanDeleteList[mail.id] = mail
				end
				self.m_mailUnReadList[mail.id] = nil
			end
		end

		if #itemList > 0 then
			self:notifyObservers("receiveMail",{itemList = itemList})
		end

		if table.count(self.m_mailUnReceiveList) <= 0  and table.count(self.m_mailUnReadList) <= 0 then
			EventDispatcher:addMsg("on_menu_visible_change", {name="email", value = false})
		end
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end