local This, super = class(..., "NodeUI")
This.fileName = "MenuScene/NodeUI/DailyQuest.csb"

local ItemNode = require("UI.ItemNode")
local GamePlayer = require("Player.GamePlayer")
local MissionConfig = require("Metadatas.MissionConfig")

function This:initData()
    self.m_manager = require("Quest.QuestManager")
    self.m_refreshDiamondCost = 0
end

function This:initNode()
    self:updateAchieveCount()
    self:updateRefreshCountDown()
    self:updateRefreshButton()
    self:updateQuestScrollView()
    self:initTimer()
end

function This:initListener()
    self.m_manager:addObserver(self, {
        dataUpdate = function(self)
            self:updateQuestScrollView()
        end,
        startDailyQuest = function(self)
            self:updateAchieveCount()
            self:updateQuestScrollView()
        end,
        achieveDailyQuest = function(self)
            self:updateQuestScrollView()
        end,
        finishDailyQuest = function(self, data)
            local itemList = data.item
            if itemList and #itemList > 0 then
                g_scene:showReward(itemList)
            end
            self:updateQuestScrollView()
        end,
        refreshDailyQuests = function(self, data)
            local itemList = data.item
            if itemList and #itemList > 0 then
                g_scene:showReward(itemList)
            end
            self:updateRefreshCountDown()
            self:updateRefreshButton()
            self:updateQuestScrollView()
        end,
    })
end

function This:onExit()
    self.m_manager:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
    self.refresh:setTouchEnded(function()
        local achieveCost = 0
        local remainSecond = 0
        if self.m_runningNode then
            g_scene:showTips(Serialize[64039])
            return 
        end
        if self.m_runningNode then
            remainSecond = self.m_runningNode.quest.totalSecond - (GamePlayer:getServerTime() - self.m_runningNode.quest.startTime)
            achieveCost = self.m_manager:calcAchieveDiamondCost(remainSecond)
        end
        local finalCost = self.m_refreshDiamondCost + achieveCost
        if finalCost > 0 then
            if remainSecond > 0 and remainSecond <= 5 then
                g_scene:showTips(Serialize[64040])
            else
                local params = {cost = finalCost, okHandler = function()
                    self.m_manager:refreshDailyQuests()
                end}
                if achieveCost > 0 then 
                    local showTime = remainSecond + self.m_manager:getRemainRefreshSecond()
                    local hour = math.floor(showTime / 3600)
                    local minute = math.floor(showTime % 3600 / 60)
                    local second = math.floor(showTime % 60)
                    params.note = string.format(Serialize[64041], hour, minute, second)
                    params.desc = Serialize[64042]
                else
                    params.desc = Serialize[60115]
                    params.notips = "DailyQuest"
                end
                local panel = g_scene:openDialog("UI.CostConfirmPanel", params)
            end
        else
            self.m_manager:refreshDailyQuests()
        end
    end)
end

function This:initTimer()
    local lastTime = GamePlayer:getServerTime()
    local allPassTime = 0
    local updateFunc = function(self, _, delay)
        local currentTime = GamePlayer:getServerTime()
        local passTime = currentTime - lastTime
        if passTime > 0 then
            allPassTime = allPassTime + passTime
            self:updateRefreshCountDown()
            self:updateRefreshButton()
            self:updateQuestCountDown()
        end
        lastTime = currentTime
    end
    self:setTimer(1, updateFunc, "MainLoop", -1)
end

function This:updateAchieveCount()
    local remain = self.m_manager:getRemainAchieveCount()
    local total = self.m_manager:getTotalAchieveCount()
    self.refreshCount:setString(remain .. "/" .. total)
    local hasRemain = remain > 0 
    self.refreshCount:setVisible(hasRemain)
    self.bg6:setVisible(hasRemain)
    self.count:setVisible(hasRemain)
    
end                              

function This:updateRefreshCountDown()
    local remainSecond = self.m_manager:getRemainRefreshSecond()
    local isVisible = (remainSecond > 0) and self.m_manager:hasRemainAchieveCount()
    self.timedes:setVisible(isVisible)
    self.time:setVisible(isVisible)
    self.bg5:setVisible(isVisible)
    if not isVisible then
        return
    end
    local hour = math.floor(remainSecond / 3600)
    local minute = math.floor(remainSecond % 3600 / 60)
    local second = math.floor(remainSecond % 60)
    self.time:setString(string.format("%02d:%02d:%02d", hour, minute, second))
end

function This:updateRefreshButton()
    local canRefresh = self.m_manager:hasRemainAchieveCount()
    self.refresh:setVisible(canRefresh)
    if not canRefresh then
        self.m_refreshDiamondCost = 0
            self.empty:setString(Serialize[60114])
            self.emptyNote:setVisible(true)
        return
    end
    local remainSecond = self.m_manager:getRemainRefreshSecond()
    local diamondCost = self.m_manager:calcRefreshDiamondCost(remainSecond)
    self.m_refreshDiamondCost = diamondCost
    self.refresh.value:setString(diamondCost)
    local isVisible = diamondCost > 0
    self.refresh.icon:setVisible(isVisible)
    self.refresh.value:setVisible(isVisible)
    self.refresh.text:setVisible(isVisible)
    self.refresh.text_free:setVisible(not isVisible)
end

function This:updateQuestScrollView()
    self.scrollView:removeAllItems()
    self.scrollView:setTouchEnabled(true)
    local questList = self.m_manager:getDailyQuestList()
    local runningNode = nil
    for i, quest in ipairs(questList) do
        local node = cc.Node:createFromCSFile("MenuScene/NodeUI/TaskItem.csb")
        node.tasksituation1.taskbegin:setTouch({
            isContainer = true,
            onEnded = function()
                self.m_manager:startDailyQuest(node.quest.id)
            end,
        })
        node.tasksituation1.taskfinish:setTouch({
            isContainer = true,
            onEnded = function()
                self.m_manager:finishDailyQuest(node.quest.id)
            end,
        })
        node.tasksituation2.accelerate:setTouch({
            isContainer = true,
            onEnded = function()
                local remainSecond = node.quest.totalSecond - (GamePlayer:getServerTime() - node.quest.startTime)
                local diamondCost = self.m_manager:calcAchieveDiamondCost(remainSecond)
                if diamondCost > 0 then
                    if remainSecond > 5 then 
                        local hour = math.floor(remainSecond / 3600)
                        local minute = math.floor(remainSecond % 3600 / 60)
                        local second = math.floor(remainSecond % 60)
                        questID = node.quest.id
                        local panel = g_scene:openDialog("UI.CostConfirmPanel", {cost = diamondCost, desc = Serialize[60112], note = string.format(Serialize[64041], hour, minute, second)})
                        panel:setOkHandler(function ()
                            self.m_manager:achieveDailyQuest(questID)
                        end)
                    elseif remainSecond > 0 then
                        g_scene:showTips(Serialize[60102])
                    end
                end
            end,
        })
        self:resetQuest(node, quest)
        self.scrollView:pushBackCustomItem(node, 710, 100)
        if quest.state == 2 then
            runningNode = node
        end
    end
    self.m_runningNode = runningNode
    self.scrollView:runItemAppearAction()

    if not questList or #questList <= 0 then
        self.empty:setVisible(true)
    else
        self.empty:setVisible(false)
    end 
end

local function updateQuestCountDown(node)
    local quest = node.quest
    local bgNode = node.tasksituation2.processBG
    local totalSecond = quest.totalSecond
    local showSeceond = GamePlayer:getServerTime() - quest.startTime
    if showSeceond > totalSecond then
        showSeceond = totalSecond
    end
    bgNode.process:setRatio(showSeceond / totalSecond)
end


local function resetQuest(node, quest)
    if quest then
        node.quest = quest
    else
        quest = node.quest
    end

    local itemList = quest.item
    for i = 1, 3 do
        local item = itemList[i]
        local itemNode = node["item" .. i]
        itemNode:setVisible(item ~= nil)
        if item then
            if not itemNode.node then
                itemNode.node = ItemNode:create(item.id, item.num)
                itemNode.node:setAnchorPoint(0.5, 0.5)
                itemNode:addChild(itemNode.node)
            else
                itemNode.node:init(item.id, item.num)
            end
        end
    end

    local quality = MissionConfig[quest.id].quality or 1
    node.title:setString(quest.title)
    local color = QualityColor[quality]
    node.title:setColor(color.r,color.g,color.b)
    node.quality:setImage("MenuScene/taskqulity" .. quality .. ".png")
    node.desc:setString(quest.desc)
    
    local isRunning = quest.state == 2
    node.tasksituation2:setVisible(isRunning)
    node.tasksituation1:setVisible(not isRunning)
    if isRunning then
        updateQuestCountDown(node)
    else
        local isNormal = quest.state == 1
        node.tasksituation1.taskbegin:setVisible(isNormal)
        node.tasksituation1.taskfinish:setVisible(not isNormal)
        if isNormal then
            local totalSecond = quest.totalSecond
            local hour = math.floor(totalSecond / 3600)
            local minute = math.floor(totalSecond % 3600 / 60)
            local second = math.floor(totalSecond % 60)
            node.tasksituation1.taskbegin.time:setString(string.format("%02d:%02d:%02d", hour, minute, second))
        end
    end
end

function This:resetQuest(node, quest)
    node.reset = resetQuest
    node:reset(quest)
    node.updateCountDown = updateQuestCountDown
end

function This:updateQuestCountDown()
    if self.m_runningNode then
        self.m_runningNode:updateCountDown()
    end
end