local This = class(...)
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GamePlayer = require("Player.GamePlayer")

local STATE_NORMAL = 1
local STATE_RUNNING = 2
local STATE_ACHIEVE = 3
local STATE_FINISH = 4

local kRefreshInterval = 3600

function This:init()
    self:initData()
    self.m_observers = {}
    self:initListener()
end

function This:initData()
    self.m_data = {
        -- dailyQuests = {},
        -- totalAchieveCount = 0,
        -- achieveCount = 0,
        -- refreshTime = 0,
        -- refreshSecondInterval = 0,
    }
end

function This:initListener()
    EventDispatcher:addListener(self, "init_role_info", self.loadData)
    EventDispatcher:addListener(self, 10500, function(self, message)
        local isUpdate = next(self.m_data) ~= nil
        self.m_data = message
        if isUpdate then
            self:callObservers("dataUpdate")
        else
            self:callObservers("loadData")
        end

        local idList = {}
        for k,quest in pairs(self.m_data.dailyQuests) do
            if quest.state > 2 then
                table.insert( idList, quest.id )
            end
        end
        EventDispatcher:addMsg("common_reddot_multi", {params = {isAll = true}, taskDaily = idList})
		EventDispatcher:dispatcher("dialy_achieve_changed")
    end)
    EventDispatcher:addListener(self, 10501, function(self, message)
        if message.err_code == 0 then
            self:callObservers("startDailyQuest")
        else
            self:handleErrorCode(message.err_code)
        end
    end)
    EventDispatcher:addListener(self, 10504, function(self, message)
        if message.err_code == 0 then
            self:callObservers("finishDailyQuest", message)
        else
            self:handleErrorCode(message.err_code)
        end
    end)
    EventDispatcher:addListener(self, 10503, function(self, message)
        if message.err_code == 0 then
            self:callObservers("refreshDailyQuests", message)
            EventDispatcher:addMsg("common_reddot_multi", {params = {isAll = true}, taskDaily = {}})
			EventDispatcher:dispatcher("dialy_achieve_changed")
        else
            self:handleErrorCode(message.err_code)
        end
    end)
    EventDispatcher:addListener(self, 10502, function(self, message)
        if message.err_code == 0 then
            self:callObservers("achieveDailyQuest", message)
        else
            self:handleErrorCode(message.err_code)
        end
    end)
end

function This:removeListener()
    EventDispatcher:removeListener(self,"init_role_info")
    EventDispatcher:removeListener(self, 10500)
    EventDispatcher:removeListener(self, 10501)
    EventDispatcher:removeListener(self, 10502)
    EventDispatcher:removeListener(self, 10503)
    EventDispatcher:removeListener(self, 10504)
end

function This:handleErrorCode(errorCode)
    g_scene:handleErrorCode(errorCode)
end

function This:addObserver(observer, callbacks)
    self.m_observers[observer] = callbacks or observer
end

function This:removeObserver(observer)
    self.m_observers[observer] = nil
end

function This:callObservers(handlerName, ...)
    for observer, callbacks in pairs(self.m_observers) do
        local callback = callbacks[handlerName]
        if callback then
           callback(observer, ...)
        end
    end
end

function This:hasData()
    return next(self.m_data) ~= nil
end

function This:loadData()
    Socket:send(10500)
end

function This:getData()
    return self.m_data
end

function This:getDailyQuestByID(id)
    for i, quest in ipairs(self.m_data.dailyQuests) do
        if quest.id == id then
            return quest
        end
    end
end

local function questSortComp(quest1, quest2)
    if quest1.state ~= quest2.state then
        return quest1.state > quest2.state
    else
        return quest1.id > quest2.id
    end
end
function This:getDailyQuestList()
    local questList = {}
    for k,quest in pairs(self.m_data.dailyQuests) do
        table.insert(questList, quest)
    end
    if #questList > 1 then
        table.sort(questList, questSortComp)
    end
    for i, quest in ipairs(questList) do
        print(tostring(i), tostring(quest.id), tostring(quest.title), tostring(quest.state))
    end
    return questList
end

function This:getTotalAchieveCount()
    return self.m_data.totalAchieveCount
end

function This:getRemainAchieveCount()
    local achieveCount = self.m_data.achieveCount
    return self:getTotalAchieveCount() - achieveCount
end

function This:getRemainRefreshSecond()
    local lastTime = self.m_data.refreshTime
    if lastTime <= 0 then
        return 0
    end
    local refreshSecondInterval = kRefreshInterval
    local remainSecond = refreshSecondInterval - (GamePlayer:getServerTime() - lastTime)
    if remainSecond > 0 then
        return remainSecond
    else
        return 0
    end
end

function This:calcRefreshDiamondCost(remainSecond)
    if remainSecond > 0 then
        local diamond = math.ceil(0.015 * (remainSecond ^ 0.8))
        return math.max(diamond, 10)
    else
        return 0
    end
end

function This:calcAchieveDiamondCost(remainSecond)
    if remainSecond > 0 then
        local diamond = math.ceil(0.015 * (remainSecond ^ 0.8))
        return math.max(diamond, 1)
    else
        return 0
    end
end

function This:hasRunningQuest()
    for k,quest in pairs(self.m_data.dailyQuests) do
        if quest.state == STATE_RUNNING then
            return true
        end
    end
    return false
end

function This:hasFinishQuest()
	if self.m_data.dailyQuests then
		for k,quest in pairs(self.m_data.dailyQuests) do
			if quest.state > STATE_RUNNING then
				return true
			end
		end
	end
    return false
end

function This:startDailyQuest(id)
    local quest = self:getDailyQuestByID(id)
    if not quest or quest.state > STATE_NORMAL then
        self:handleErrorCode("invalidAction")
        return
    end
    if self:hasRunningQuest() then
        self:handleErrorCode("hasRunningQuest")
        return
    end
    if not self:hasRemainAchieveCount() then
        self:handleErrorCode("achieveCountLack")
        return
    end
    Socket:sendAndLock(10501, {id=id})
end

function This:finishDailyQuest(id)
    local quest = self:getDailyQuestByID(id)
    if not quest or quest.state ~= STATE_ACHIEVE then
        self:handleErrorCode("invalidAction")
        return
    end
    Socket:sendAndLock(10504, {id=id})
end

function This:hasRemainAchieveCount()
    return self.m_data.totalAchieveCount > self.m_data.achieveCount
end

function This:refreshDailyQuests()
    if not self:hasRemainAchieveCount() then
        self:handleErrorCode("invalidAction")
        return
    end
    Socket:sendAndLock(10503)
end

function This:achieveDailyQuest(id)
    local quest = self:getDailyQuestByID(id)
    if not quest or quest.state ~= STATE_RUNNING then
        self:handleErrorCode("invalidAction")
        return
    end
    local remainSecond = quest.totalSecond - (GamePlayer:getServerTime() - quest.startTime)
    if remainSecond > 0 then
        Socket:sendAndLock(10502, {id=id})
    end
end