local This , super = class(..., "Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local VipMemberModel = require("Vip.VipMemberModel")
local VipUtil = require("Vip.VipUtil")
local MatchingManager = require("Match.MatchManager")

function This:init()
    super.init(self)
 
    self.m_inited = false
    self.m_accMoney = 0
    self.m_members = {}

    for i=VipType.kNone, VipType.kMax do
        local model = VipMemberModel:create({id=i})
        self.m_members[i] = model
    end

    self:initListener()
end

function This:canShowWarming()
    local ret = true
    local GamePlayer = require("Player.GamePlayer")
    local OpenLevelConfig = require("Metadatas.OpenLevelConfig")

    ret = GamePlayer:getLevel() >= (OpenLevelConfig[18].open_level or 0)
    if not ret then 
        return false
    end

    return true
end

function This:initListener()
    EventDispatcher:addListener(self, "vip_end", self.onGetVipInfoRequest)
    EventDispatcher:addListener(self, 11700, self.onGetVipInfoReply)
    EventDispatcher:addListener(self, 11701, self.onGetVipRewardReply)
end

function This:removeListener()
    EventDispatcher:removeListener(self, "vip_end")
    EventDispatcher:removeListener(self, 11700)
    EventDispatcher:removeListener(self, 11701)
end

function This:getNewestModel()
    local ret = nil
    for level, model in pairs(self.m_members) do
        if model:isUpdating() then
            if not ret or model:getLeftTime() >= ret:getLeftTime() then
                ret = model
            end
        end
    end
    return ret
end

function This:getMemberState()
    local  state = {
        [VipType.kNone] = true,
        [VipType.kVIP] = self.m_members[VipType.kVIP]:isUpdating(),
        [VipType.kSVIP] = self.m_members[VipType.kSVIP]:isUpdating(),
        [VipType.kSSVIP] = self.m_members[VipType.kSSVIP]:isUpdating(),
    }
    return state
end

function This:getMemberStateBy( vipType )
    local  state = self:getMemberState()
    return state[vipType]
end

function This:getMaxMemberLevel()
    local maxLevel = VipType.kNone
    local stateMap = self:getMemberState()
    for level, state in pairs(stateMap) do
        if state and maxLevel < level then
            maxLevel = level
        end
    end
    return maxLevel
end

function This:isFullMember()
    local isFull = true
    local stateMap = self:getMemberState()
    for _, state in pairs(stateMap) do
        isFull = isFull and state
        if not isFull then
            break
        end
    end
    return isFull
end

function This:hasPrivilege( privilegeType, withWarming )
    local model
    for i = VipType.kVIP, VipType.kMax do
        local curModel = self.m_members[i]
        if curModel:getPrivilegeNumBy( privilegeType ) > 0 then
            model = curModel
            break
        end
    end

    if model then
        result = model:hasPrivilege(privilegeType)
    else
        return false, nil
    end

    if withWarming and not result then
        self:showVipWarming( privilegeType, model:getName() )
    end
    return result, model
end

function This:showVipWarming( privilegeType, name )
    if not self:canShowWarming() then
        return false
    end

    if self:isFullMember() then
        g_scene:showTips(Serialize["vip_priv_fulltip_" .. privilegeType])
        return true
    end
	local formatStr = Serialize["vip_priv_warming_" .. privilegeType]
    local des = string.format(formatStr, name)
    local panel = g_scene:openDialog("UI.ConfirmPanel", {
            fileName = "MenuScene/NodeUI/MemberActivation.csb" , 
            text = des, 
            okBtnTxt = Serialize[62234],
            okHandler = function () g_scene:openDialog("Vip.VipPanel") end
    })
    return true
end

function This:getPrivilegeNumBy( privilegeType )
    local value = 0

    for i = VipType.kNone, VipType.kMax do
        local model = self.m_members[i]
        local isValid = model:hasPrivilege(privilegeType)
        if isValid then
            value = value + model:getPrivilegeNumBy(privilegeType)
        end
    end
    return value
end

function This:isPrivilegeNumEnough( privilegeType, targetNum, withWarming )
    local result = false
    result = self:getPrivilegeNumBy( privilegeType ) > targetNum
    if withWarming and not result then
        self:showVipWarming( privilegeType, "")
    end
    return result
end

function This:getMemberModels()
    return self.m_members
end

function This:addVipMemberModel( info )
    local model = self.m_members[info.id]
    if not model then
        model = VipMemberModel:create(info)
        self.m_members[info.id] = model
    else
        model:updateInfo(info)
    end
    return model
end

function This:getCurPrice()
    return self.m_accMoney
end

function This:setNewPrice( money )
    local oldValue = self.m_accMoney
    self.m_accMoney = money
    if self.m_inited then
        self:notifyObservers("onVipActivated", self:getMaxMemberLevel())
    end
end

----------------------------------------------
function This:onGetVipInfoRequest()
    Socket:sendAndLock(11700)
end

function This:onGetVipInfoReply( msg )
    if msg then
        local activeList, dailyList = {}, {}
        for index, info in pairs(msg.vip_list) do
            local model = self:addVipMemberModel(info)
            if model:isUpdating() and model:hasFirstReward() then
                table.insert(activeList, model:getId() * 10 + VipReward.kActive)
            end
            if model:isUpdating() and model:hasDailyReward() then
                table.insert(dailyList, model:getId() * 10 + VipReward.kDaily)
            end
        end
        self:setNewPrice( msg.price )
        self.m_inited = true
        self:notifyObservers("onGetVipInfo")
        EventDispatcher:addMsg("vip_reward_state_changed")
    else
        g_scene:handleErrorCode("")
    end
end

function This:hasVipReward()
    local ret = false
    if self.m_members then
        for i, model in pairs(self.m_members) do
            if model:isUpdating() and model:hasFirstReward() then
                ret = true
                break
            end
            if model:isUpdating() and model:hasDailyReward() then
                ret = true
                break
            end
        end
    end
    return ret
end

function This:onGetVipRewardRequest( vipType, rewardType )
    Socket:sendAndLock(11701, { id = vipType, type = rewardType })
end

function This:onGetVipRewardReply( msg )
    if msg then
        if msg.err_code == 0 then
            local model = self.m_members[msg.id]
            model:setRewardState(msg.type)
            self:notifyObservers("onGetVipReward", msg, model)
            EventDispatcher:addMsg("vip_reward_state_changed")
        else
            g_scene:handleErrorCode(msg.err_code)
        end
    end
end