-- @Author: superm
-- @Date:   2017-10-21 10:51:07
-- @Last Modified by:   superm
-- @Last Modified time: 2018-08-16 10:51:22
-- @File Name: VipMemberModel.lua
-- @Description:

local This , super = class(...,"Base.BaseModel")
local GamePlayer = require("Player.GamePlayer")
local VipConfig = require("Metadatas.VipConfig")          
local EventDispatcher = require("EventDispatcher")          

function This:init( info )
    super.init(self)

    self.m_view = nil
    if info then
        self:initSelf( info )

        self.m_state = VipState.kLocked
        self.m_upStartTime = info.start_time or -1
        self.m_upEndTime = info.end_time or -1
        self.m_upEscape = 0
        self.m_upIsUpdating = false
        if self.m_ID > 0 then
            self:updateState(true, info)
        else
            self.m_state = VipState.kUpdating
        end
    else
        self.m_ID = 0
        self.m_upStartTime = -1
        self.m_upEndTime = -1
        self.m_upEscape = 0
        self.m_upIsUpdating = false

        self.m_state = VipState.kNormal
        self.m_hasActiveReward = false
        self.m_hasDailyReward = false
        self.m_hasRenewReward = false
    end
end

function This:initIdByInfo( info )
    self.m_ID = info.id
    self.m_config = VipConfig[self.m_ID] 
end

function This:initSelf( info )
    self:initIdByInfo(info)
    self.m_hasActiveReward = info.active_reward or false
    self.m_hasDailyReward = info.daily_reward or false
    self.m_hasRenewReward = info.renewal_reward or false
end

function This:updateSelf( info )
    self.m_hasActiveReward = info.active_reward
    self.m_hasDailyReward = info.daily_reward
    self.m_hasRenewReward = info.renewal_reward or false
end

function This:updateInfo( info )
    local isDiff = not self:isSameType(info) 
    self.m_upStartTime = info.start_time or 0
    self:setEndTime(info.end_time or 0)
    self:updateSelf( info )
    self:updateState(isDiff, info)
end

function This:updateState(isInit, info)
    local isChanged = false
    local oldState = self.m_state
    if isInit then
        self:initSelf(info)
        isChanged = true
    end

    local curTime = GamePlayer:getServerTime()
    if curTime < self.m_upEndTime then
        self.m_state = VipState.kUpdating
        self:startUpdate()
    elseif curTime >= self.m_upEndTime then
        self.m_state = VipState.kLocked
        self:endUpdate()
    end

    if not isChanged and self.m_state ~= oldState then
        isChanged = true
    end

    return isChanged
end

function This:startUpdate()
    if not self.m_upIsUpdating then
        cc.Application:addToUpdateList(self)
        self:notifyObservers("onVipUpdateStart", self)
        self.m_upIsUpdating = true
    end
end

function This:endUpdate()
    if self.m_upIsUpdating then
        cc.Application:removeFromUpdateList(self)
        self:notifyObservers("onVipUpdateEnd", self)
        EventDispatcher:dispatcher("vip_end")
        self.m_upIsUpdating = false
    end
end

function This:update( dt )
    self.m_upEscape = self.m_upEscape + dt
    if self.m_upEscape >= 1.0 then
        self.m_upEscape = self.m_upEscape - 1
        local leftTime = self:getLeftTime()
        if leftTime > 0 then
            self:notifyObservers("updateVipTime", {id = self:getId() , time = leftTime})

        else
            self:notifyObservers("updateVipTime", {id = self:getId() , time = 0})
            self:endUpdate()
            self.m_state = VipState.kLocked
        end
    end
end

function This:setEndTime( endTime )
    self.m_upEndTime = endTime
    local curTime = GamePlayer:getServerTime()
    if curTime < self.m_upEndTime then
        self.m_state = TerrObjectState.kUpdating
        self:startUpdate()
    end
    self:notifyObservers("onInfoChanged", self)
end

function This:setId( id )
    self.m_ID = id
end

function This:getId()
    return self.m_ID
end

function This:getName()
    return VipName[self.m_ID]
end

function This:getConfigId()
    return self.m_config.id
end

function This:getLeftTime()
    local curTime = GamePlayer:getServerTime()
    if curTime >= self.m_upEndTime then
        return 0
    end
    return self.m_upEndTime - curTime
end

function This:getDays( )
   local days = math.secondToDay(self:getLeftTime())
   return days
end

function This:isLocked()
    return self.m_state == VipState.kLocked
end

function This:isUpdating()
    return self.m_state == VipState.kUpdating
end

function This:isSameType( info )
    return true
end

function This:isQualified()
    local costList = self.m_nextConfig.upcost
    if not costList or costList == 0 then
        return true
    end

    local ret = true
    if not isNestedObject(costList) then
        if #costList >= 2 then
            ret = self:isCostEnoughBy(costList[1], costList[2])
        end
    else
        for _, cond in ipairs(costList) do
            ret = ret and self:isCostEnoughBy(cond[1], cond[2])
            if not ret then
                return false
            end
        end
    end
    return ret
end

function This:isMaxLevel()
    return self:getId() >= VipType.kMax
end

function This:hasPrivilege( privilegeType )
    return self:isUpdating()
end

function This:hasFirstReward( )
    return self.m_hasActiveReward
end

function This:hasDailyReward( )
    return self.m_hasDailyReward
end

function This:hasRenewReward( )
    return self.m_hasRenewReward
end

function This:hasFirstRecharge()
    return self.m_upEndTime <= 0
end

function This:setRewardState( rewardType )
    if rewardType == VipReward.kActive then
        self.m_hasActiveReward = false
    elseif rewardType == VipReward.kDaily then
        self.m_hasDailyReward = false
    elseif  rewardType == VipReward.kRenew then
        self.m_hasRenewReward = false
    end
end

--获取配置信息region begin

function This:getConfig()
    return self.m_config
end

function This:getPrivilegeNumBy(privilegeType)
    local privilege = self.m_config[ privilegeType ]
    return privilege or 0
end

function This:getConfigPrice( )
    return self.m_config.count
end

function This:getMaxPrice( )
    local maxValue = self:getConfigPrice()
    for _, cfg in ipairs(VipConfig) do
        if maxValue < cfg.count then
            maxValue = cfg.count
        end
    end
    return maxValue
end

--获取配置信息region end

--endregion