local ffi = require("ffi")
ffi.cdef[[
typedef struct b2BodyDef
{
    int32 type;
    float32 x;
    float32 y;
    float32 angle;
    float32 vx;
    float32 vy;
    float32 angularVelocity;
    float32 linearDamping;
    float32 angularDamping;
    int32 allowSleep;
    int32 awake;
    int32 fixedRotation;
    int32 bullet;
    int32 active;
    void* userData;
    float32 gravityScale;
    int32 isContinue;
    int32 isVelocityAngle;
}b2BodyDef;

typedef struct b2PointList{
    float x1,y1, x2,y2, x3,y3, x4,y4, x5,y5, x6,y6, x7,y7, x8,y8; 
}b2PointList;
]]

local This = class(...)
b2.Body = This
local b2BodyDef = ffi.typeof("b2BodyDef")
local b2PointList = ffi.typeof("b2PointList")
b2.PointList = b2PointList
local Vec = cc.Vec
local b2Shape = b2.Shape
local STATIC_TYPE = 0
local KINEMATIC_TYPE = 1
local DYNAMIC_TYPE = 2
local ANCHOR_CENTER = Vec(0.5,0.5)
local VEC_ZEOR = Vec(0, 0)
local lib_b2Body = lib2d.b2Body
local setmetatable = setmetatable

b2.BODY_TYPE_DYNAMIC = DYNAMIC_TYPE
b2.BODY_TYPE_STATIC = STATIC_TYPE
b2.BODY_TYPE_KINEMATIC = KINEMATIC_TYPE


local function StaticBodyDef(t)
    local def = b2BodyDef()
    def.allowSleep = 1
    def.awake = 1
    def.type = 0
    def.active = 1
    def.gravityScale = 1
    if t then
        for k,v in pairs(t) do
            def[k] = v
        end
    end
    return def
end

local function DynamicBodyDef(t)
    local def = b2BodyDef()
    def.allowSleep = 1
    def.awake = 1
    def.type = 2
    def.active = 1
    def.gravityScale = 1
    if t then
        for k,v in pairs(t) do
            def[k] = v
        end
    end
    return def
end

b2.StaticBodyDef = StaticBodyDef
b2.DynamicBodyDef = DynamicBodyDef
b2.BodyDef = b2BodyDef

local lib_addPolygonShape = lib_b2Body.addPolygonShape
function This:addPolygonShape(shapeDef, pointList)
    local shapePtr = lib_addPolygonShape(self[0], shapeDef, b2PointList(pointList), #pointList / 2)
    local shape = {[0] = shapePtr, m_body = self, m_owner = false, m_category = shapeDef.categoryBits}
    setmetatable(shape, b2Shape)
    self.m_shapes[shapePtr] = shape
    self.m_physics.m_shapes[shapePtr] = shape
    return shape
end

local lib_addBoxShape = lib_b2Body.addBoxShape
function This:addBoxShape(shapeDef, width, height, anchor, offset)
    anchor = anchor or ANCHOR_CENTER
    offset = offset or VEC_ZEOR
    local shapePtr = lib_addBoxShape(self[0], shapeDef, width, height, anchor.x, anchor.y, offset.x, offset.y)
    local shape = {[0] = shapePtr, m_body = self, m_owner = false, m_category = shapeDef.categoryBits}
    setmetatable(shape, b2Shape)
    self.m_shapes[shapePtr] = shape
    self.m_physics.m_shapes[shapePtr] = shape
    return shape
end

function This:addBoxShape2(shapeDef, width, height, anchorX, anchorY, offsetX, offsetY)
    anchorX = anchorX or 0.5
    anchorY = anchorY or 0.5
    offsetX = offsetX or 0
    offsetY = offsetY or 0
    local shapePtr = lib_addBoxShape(self[0], shapeDef, width, height, anchorX, anchorY, offsetX, offsetY)
    local shape = {[0] = shapePtr, m_body = self, m_owner = false, m_category = shapeDef.categoryBits}
    setmetatable(shape, b2Shape)
    self.m_shapes[shapePtr] = shape
    self.m_physics.m_shapes[shapePtr] = shape
    return shape
end

function b2.getBoxShapeBorder(info, width, height, anchor, offset)
    anchor = anchor or ANCHOR_CENTER
    offset = offset or VEC_ZEOR
    local x = -anchor.x * width + offset.x
    local y = -anchor.y * height + offset.y
    info.leftX = x
    info.leftY = y + height
    info.rightX = x + width
    info.rightY = y + height
    info.minY = y
end

function b2.getPolygonBorder(info, pointList)
    local leftX, leftY = pointList[1], pointList[2]
    local rightX, rightY = leftX, leftY
    local minY = leftY
    for i=1, #pointList, 2 do
        local x = pointList[i]
        local y = pointList[i + 1]
        if x < leftX then
            leftX = x
            leftY = y
        end
        if x > rightX then
            rightX = x
            rightY = y
        end
        if y < minY then
            minY = y
        end
    end
    info.leftX = leftX
    info.leftY = leftY
    info.rightX = rightX
    info.rightY = rightY
    info.minY = minY
end

local lib_addCircleShape = lib_b2Body.addCircleShape
function This:addCircleShape(shapeDef, radius, centerX, centerY)
    local shapePtr = lib_addCircleShape(self[0], shapeDef, radius, centerX or 0, centerY or 0)
    local shape = {[0] = shapePtr, m_body = self, m_owner = false, m_category = shapeDef.categoryBits}
    setmetatable(shape, b2Shape)
    self.m_shapes[shapePtr] = shape
    self.m_physics.m_shapes[shapePtr] = shape
    return shape
end

local lib_addEdgeShape = lib_b2Body.addEdgeShape
function This:addEdgeShape(shapeDef, x1, y1, x2, y2)
    local shapePtr = lib_addEdgeShape(self[0], shapeDef, x1, y1, x2, y2)
    local shape = {[0] = shapePtr, m_body = self, m_owner = false, m_category = shapeDef.categoryBits}
    setmetatable(shape, b2Shape)
    self.m_shapes[shapePtr] = shape
    self.m_physics.m_shapes[shapePtr] = shape
    return shape
end

function This:addSegmentShape(shapeDef, x1, y1, x2, y2)
    return self:addEdgeShape(shapeDef, x1, y1, x2, y2)
end

function This:destroy()
    self.m_owner = nil
    local physics = self.m_physics
    self.m_physics = nil
    for id,shape in pairs(self.m_shapes) do
        shape.m_owner = nil
        shape.m_body = nil
        physics.m_shapes[id] = nil
    end
    self.m_shapes = nil
    physics:destroyBody(self)
end

local lib_removeShape = lib_b2Body.removeShape
function This:removeShape(shape)
    shape.m_owner = nil
    shape.m_body = nil
    self.m_shapes[shape[0]] = nil
    self.m_physics.m_shapes[shape[0]] = nil
    lib_removeShape(self[0], shape[0])
end

local lib_getType = lib_b2Body.getType
function This:getType()
    return lib_getType(self[0])
end

local lib_getPosition = lib_b2Body.getPosition
function This:getPosition()
    return lib_getPosition(self[0])
end

local lib_setPosition = lib_b2Body.setPosition
function This:setPosition(x, y)
    lib_setPosition(self[0], x, y)
end

local lib_setXY = lib_b2Body.setXY
function This:setXY(x, y)
    lib_setXY(self[0], x, y)
end

local lib_addXY = lib_b2Body.addXY
function This:addXY(x, y)
    if x ~= 0 or y ~= 0 then
        lib_addXY(self[0], x, y)
    end
end

local lib_getAngle = lib_b2Body.getAngle
function This:getAngle()
    return lib_getAngle(self[0])
end

local lib_setAngle = lib_b2Body.setAngle
function This:setAngle(angle)
    lib_setAngle(self[0], angle)
end

local lib_setAngleVetor = lib_b2Body.setAngleDirection
local deg = math.deg
local atan2 = math.atan2
function This:setAngleVetor(vx, vy)
    local angle = deg(atan2(vy, vx))
    lib_setAngle(self[0], angle)
end

local lib_setTransform = lib_b2Body.setTransform
function This:setTransform(x, y, angle)
    lib_setTransform(self[0], x, y, angle)
end

local lib_setIsActive = lib_b2Body.setIsActive
function This:setIsActive(isActive)
    lib_setIsActive(self[0], isActive)
end

local lib_isActive = lib_b2Body.isActive
function This:isActive()
    return lib_isActive(self[0])
end

local lib_getWorldCenter = lib_b2Body.getWorldCenter
function This:getWorldCenter()
    return lib_getWorldCenter(self[0])
end

local lib_setVelocity = lib_b2Body.setVelocity
function This:setVelocity(x, y)
    lib_setVelocity(self[0], x, y)
end

local lib_getVelocity = lib_b2Body.getVelocity
function This:getVelocity()
    return lib_getVelocity(self[0])
end

local lib_addVelocity = lib_b2Body.addVelocity
function This:addVelocity(vx, vy)
    lib_addVelocity(self[0], vx, vy)
end

local lib_setAngularVelocity = lib_b2Body.setAngularVelocity
function This:setAngularVelocity(omega)
    lib_setAngularVelocity(self[0], omega)
end

local lib_getAngularVelocity = lib_b2Body.getAngularVelocity
function This:getAngularVelocity()
    return lib_getAngularVelocity(self[0])
end

local lib_getGravityScale = lib_b2Body.getGravityScale
function This:getGravityScale()
    return lib_b2Body.getGravityScale(self[0])
end

local lib_setGravityScale = lib_b2Body.setGravityScale
function This:setGravityScale(scale)
    lib_setGravityScale(self[0], scale)
end

local lib_setIsAwake = lib_b2Body.setIsAwake
function This:activate()
    lib_setIsAwake(self[0], true)
end

function This:setIsAwake(isAwake)
    lib_setIsAwake(self[0], isAwake)
end

local lib_isAwake = lib_b2Body.isAwake
function This:isAwake()
    return lib_isAwake(self[0])
end

local lib_setIsFixedRotation = lib_b2Body.setIsFixedRotation
function This:setIsFixedRotation(isFixed)
    return lib_setIsFixedRotation(self[0], isFixed)
end

local lib_isFixedRotation = lib_b2Body.isFixedRotation
function This:isFixedRotation()
    return lib_isFixedRotation(self[0])
end

local lib_setGravity = lib_b2Body.setGravity
function This:setGravity(gravityX, gravityY)
    lib_setGravity(self[0], gravityX, gravityY)
end

local lib_getGravity = lib_b2Body.getGravity
function This:getGravity()
    return lib_getGravity(self[0])
end

local lib_getGround = lib_b2Body.getGround
function This:getGround()
    local nx, ny, groundPtr = lib_getGround(self[0])
    local ground = nil
    if groundPtr then
        ground = self.m_physics.m_shapes[groundPtr]
    end
    return nx, ny, ground
end

function This:isStatic()
    return self.m_type == STATIC_TYPE
end

local lib_applyForceToCenter = lib_b2Body.applyForceToCenter
-- function This:applyForceToCenter(fx, fy)
function This:applyForceToCenter(fx, fy, isAwake)
    lib_b2Body.applyForceToCenter(self[0], fx, fy, isAwake == nil and true or isAwake)
end

local lib_getMass = lib_b2Body.getMass
function This:getMass()
    return lib_b2Body.getMass(self[0])
end

local lib_addContactBodyVelocity = lib_b2Body.addContactBodyVelocity
function This:addContactBodyVelocity(vx, vy)
    lib_b2Body.addContactBodyVelocity(self[0], vx, vy)
end

local lib_setLinearDamping = lib_b2Body.setLinearDamping
function This:setLinearDamping(damping)
    lib_setLinearDamping(self[0], damping)
end

local lib_resetContactFriction = lib_b2Body.resetContactFriction
function This:resetContactFriction()
    lib_resetContactFriction(self[0])
end

local lib_getWorldPoint = lib_b2Body.getWorldPoint
function This:getWorldPoint(localX, localY)
    return lib_getWorldPoint(self[0], localX, localY)
end

local lib_setIsContinue = lib_b2Body.setIsContinue
function This:setIsContinue(isContinue)
    lib_setIsContinue(self[0], isContinue)
end

local lib_rayCastDownOnGround = lib_b2Body.rayCastDownOnGround
function This:rayCastDownOnGround(x, y, range)
    return lib_rayCastDownOnGround(self[0], x, y, range)
end

function This:removeAllShape()
    for _, shape in pairs(self.m_shapes) do
        self:removeShape(shape)
    end
end

function This:setIsVelocityAngle(isVelocityAngle)
    lib_b2Body.setIsVelocityAngle(self[0], isVelocityAngle)
end

function This:disabledContact(shape)
    lib_b2Body.disabledContact(self[0], shape[0])
end