local ffi = require("ffi")
ffi.cdef[[
typedef struct ccBezierConfig {
    Vec2 endPosition;
    Vec2 controlPoint1;
    Vec2 controlPoint2;
}ccBezierConfig;
typedef struct BodyMoveAction{
    int id, easingID, vtype;
    double time, duration;
    double va0, vb0;
    double va1, vb1;
}BodyMoveAction;
typedef struct MoveAction{
    int id, easingID, vtype;
    double time, duration;
    double va0, vb0;
    double va1, vb1;
}MoveAction;
typedef struct BlurAction{
    int id, easingID, vtype;
    double time, duration;
    double v0, v1;
}BlurAction;
typedef struct MotionAction{
    int id, easingID, vtype;
    double time, duration;
    double v0, v1;
}MotionAction;
typedef struct BezierAction{
    int id, easingID, vtype;
    double time, duration;
    Vec2 v0, v1;
    Vec2 p1, p2;
}BezierAction;
typedef struct TextNumberAction{
    int id, easingID, vtype;
    double time, duration;
    int v0, v1;
}TextNumberAction;
typedef struct FadeAction{
    int id, easingID, vtype;
    double time, duration;
    int v0, v1;
}FadeAction;
typedef struct CircleAction{
    int id, easingID, vtype;
    double time, duration;
    float x, y;
    int radius;
    int angle;
    float cx, cy;
    float angle0;
    int face;
}CircleAction;
]]
local pow = math.pow
local function bezierat(a, b, c, d, t)
    return (pow(1-t,3) * a + 3*t*(pow(1-t,2))*b + 3*pow(t,2)*(1-t)*c +pow(t,3)*d)
end
local Easing = require("Easing")
local UpdateList = {}
local ActionID = {}

local ActionTypes = {
    BodyMoveAction = function (self, target)
        local easing = Easing[self.easingID]
        local x = easing(self.time, self.va0, self.va1 - self.va0, self.duration)
        local y = easing(self.time, self.vb0, self.vb1 - self.vb0, self.duration)
        target.m_body:setXY(x, y)
        target.m_position.x = x
        target.m_position.y = y
    end,
    MoveAction = function (self, target)
        local easing = Easing[self.easingID]
        local x = easing(self.time, self.va0, self.va1 - self.va0, self.duration)
        local y = easing(self.time, self.vb0, self.vb1 - self.vb0, self.duration)
        target:setPosition(x, y)
    end,
    CircleAction = function (self, target)
        local angle = Easing[self.easingID](self.time, 0, self.angle, self.duration)
        local x, y = math.rotate(self.x, self.y, self.face * angle, self.cx, self.cy)
        target.m_body:setXY(x, y)
        if self.face == -1 then
            target.m_body:setAngle(180 - (angle + self.angle0))
        else
            target.m_body:setAngle(angle + self.angle0)
        end
        
        target.m_position.x = x
        target.m_position.y = y
    end,
    BlurAction = function (self, target, v)
        target:setFactor(v)
    end,
    MotionAction = function (self, target, v)
        target.m_motionScale = v
    end,
    BezierAction = function (self, node)
        local t = self.time / self.duration
        local x = bezierat(0, self.p1.x, self.p2.x, self.v1.x, t)
        local y = bezierat(0, self.p1.y, self.p2.y, self.v1.y, t)
        node:setPosition(self.v0.x + x, self.v0.y + y)
    end,
    TextNumberAction = function (self, label, v)
        label:setString(math.floor(v))
    end,
    FadeAction = function (self, node, v)
        node:setOpacity(v)
    end,
}

local Action = {
    Easing = Easing,
    create = function (self, type, easingID, duration, v1, v2, v3, v4)
        if easingID ~= 0 then
            easingID = Easing[easingID]
        end
        if v4 then
            return self[type](ActionID[type], easingID, 4, 0, duration, v1, v2, v3, v4)
        elseif v1 then
            return self[type](ActionID[type], easingID, 2, 0, duration, v1, v2)
        else
            return self[type](ActionID[type], easingID, 0, 0, duration)
        end
    end,
    update = function (action, target, deltaTime)
        local d = action.duration
        local t = action.time
        if t >= d then
            return true
        end
        t = t + deltaTime
        if t >= d then
            t = d
        end
        action.time = t
        if action.vtype == 2 then
            local v = Easing[action.easingID](t, action.v0, action.v1 - action.v0, d)
            UpdateList[action.id](action, target, v)
        else
            UpdateList[action.id](action, target)
        end
        return t >= d
    end
}
cc.Action = Action
local actionIndex = 1
for k,v in pairs(ActionTypes) do
    Action[k] = ffi.typeof(k)
    ActionID[k] = actionIndex
    UpdateList[actionIndex] = v
    actionIndex = actionIndex + 1
end

cc.BezierConfig = ffi.typeof("ccBezierConfig")
--local config = cc.BezierConfig({endPosition=cc.Vec()})

local lib_ease = lib2d.ccEase
local easeActionList = {
    ["In"] = lib_ease.in_create,
    ["Out"] = lib_ease.out_create,
    ["InOut"] = lib_ease.inOut_create,
    ["ExponentialIn"] = lib_ease.exponentialIn_create,
    ["ExponentialOut"] = lib_ease.exponentialOut_create,
    ["ExponentialInOut"] = lib_ease.exponentialInOut_create,
    ["SineIn"] = lib_ease.sineIn_create,
    ["SineOut"] = lib_ease.sineOut_create,
    ["SineInOut"] = lib_ease.sineInOut_create,
    ["ElasticIn"] = function (action) return lib_ease.elasticIn_create(action, 0.3) end,
    ["ElasticOut"] =  function (action) return lib_ease.elasticOut_create(action, 0.3) end,
    ["ElasticInOut"] = function (action) return  lib_ease.elasticInOut_create(action, 0.3, 0.3) end,
    ["BounceIn"] = lib_ease.bounceIn_create,
    ["BounceOut"] = lib_ease.bounceOut_create,
    ["BounceInOut"] = lib_ease.bounceInOut_create,
    ["BackIn"] = lib_ease.backIn_create,
    ["BackOut"] = lib_ease.backOut_create,
    ["BackInOut"] = lib_ease.backInOut_create,
    ["QuadraticIn"] = lib_ease.quadraticActionIn_create,
    ["QuadraticOut"] = lib_ease.quadraticActionOut_create,
    ["QuadraticInOut"] = lib_ease.quadraticActionInOut_create,
    ["QuarticIn"] = lib_ease.quarticActionIn_create,
    ["QuarticOut"] = lib_ease.quarticActionOut_create,
    ["QuarticInOut"] = lib_ease.quarticActionInOut_create,
    ["QuinticIn"] = lib_ease.quinticActionIn_create,
    ["QuinticOut"] = lib_ease.quinticActionOut_create,
    ["QuinticInOut"] = lib_ease.quinticActionInOut_create,
    ["CircleIn"] = lib_ease.circleActionIn_create,
    ["CircleOut"] = lib_ease.circleActionOut_create,
    ["CircleInOut"] = lib_ease.circleActionInOut_create,
    ["CubicIn"] = lib_ease.cubicActionIn_create,
    ["CubicOut"] = lib_ease.cubicActionOut_create,
    ["CubicInOut"] = lib_ease.cubicActionInOut_create,
}
cc.Ease = easeActionList

local This, super = class(...)

cc.CallFunc = lib2d.ccAction.callFunc_create

cc.Sequence = lib2d.ccAction.sequence_create

cc.Spawn = lib2d.ccAction.spawn_create

local moveByCreate = lib2d.ccAction.moveBy_create
function cc.MoveBy(duration, x, y, easeActionName)
    local action = moveByCreate(duration, x, y)
    if easeActionName then
        action = easeActionList[easeActionName](action)
    end
    return action
end

local moveToCreate = lib2d.ccAction.moveTo_create
function cc.MoveTo(duration, x, y, easeActionName)
    local action = moveToCreate(duration, x, y)
    if easeActionName then
        action = easeActionList[easeActionName](action)
    end
    return action
end

local scaleToCreate = lib2d.ccAction.scaleTo_create
function cc.ScaleTo(duration, scale, easeActionName)
    local action = scaleToCreate(duration, scale)
    if easeActionName then
        action = easeActionList[easeActionName](action)
    end
    return action
end

function cc.ScaleXYTo(duration, scaleX, scaleY, easeActionName)
    local action = scaleToCreate(duration, scaleX, scaleY)
    if easeActionName then
        action = easeActionList[easeActionName](action)
    end
    return action
end

local scaleByCreate = lib2d.ccAction.scaleBy_create
function cc.ScaleBy(duration, scale, easeActionName)
    local action = scaleByCreate(duration, scale)
    if easeActionName then
        action = easeActionList[easeActionName](action)
    end
    return action
end


local rotateToCreate = lib2d.ccAction.rotateTo_create
function cc.RotateTo(duration, scale, easeActionName)
    local action = rotateToCreate(duration, scale)
    if easeActionName then
        action = easeActionList[easeActionName](action)
    end
    return action
end

local rotateByCreate = lib2d.ccAction.rotateBy_create
function cc.RotateBy(duration, scale, easeActionName)
    local action = rotateByCreate(duration, scale)
    if easeActionName then
        action = easeActionList[easeActionName](action)
    end
    return action
end

local sizeToCreate = lib2d.ccAction.sizeTo_create
function cc.SizeTo(duration, width, height, easeActionName)
    local action = sizeToCreate(duration, width, height)
    if easeActionName then
        action = easeActionList[easeActionName](action)
    end
    return action
end


function cc.ShakeAction()
    return cc.Sequence(
            cc.MoveBy(0.05, 10, 0),
            cc.MoveBy(0.1, -10, 0),
            cc.MoveBy(0.1, 2, 0),
            cc.MoveBy(0.1, -2, 0),
            cc.MoveBy(0.05, 0, 0)
        )
end

cc.FadeTo = lib2d.ccAction.fadeTo_create

cc.FadeIn = lib2d.ccAction.fadeIn_create

cc.FadeOut = lib2d.ccAction.fadeOut_create

cc.JumpTo = lib2d.ccAction.jumpTo_create

cc.JumpBy = lib2d.ccAction.jumpBy_create

cc.RemoveAction = lib2d.ccAction.removeAction_create

cc.DelayTime = lib2d.ccAction.delayTime_create

cc.RepeatForever = lib2d.ccAction.repeatForever_create

cc.Repeat = lib2d.ccAction.repeat_create

cc.BezierTo = lib2d.ccAction.bezierTo_create

cc.BezierBy = lib2d.ccAction.bezierBy_create

cc.Blink = lib2d.ccAction.blink_create

cc.LabelNumTo = lib2d.ccAction.labelNumTo_create

cc.Hide = lib2d.ccAction.hide_create