local This, super = class(..., "cc.Node")
cc.SkeletonNode = This

local objectRegistry = cc.objectRegistry
local registerObject = cc.registerObject
local lib_skeletonNode = lib2d.ccSkeletonNode

local spAtlas = lib2d.spAtlas
local spSkeletonData = lib2d.spSkeletonData
local function freeSkeletonData(data)
    spSkeletonData.free(data[0])
    if data.atlas then
        spAtlas.free(data.atlas)
        data.atlas = nil
    end
    if data.attachmentLoader then
        spAtlas.freeAttachmentLoader(data.attachmentLoader)
        data.attachmentLoader = nil
    end
end
local function loadSkeletonData(mainFileName, name, scale)
    local skeletonFileName= mainFileName .. "/" .. name .. "/skeleton.skel"
    local atlasFileName = mainFileName .. "/" .. name .. "/skeleton.atlas"

    local atlas = spAtlas.create(atlasFileName)
    local attachmentLoader = spAtlas.createAttachmentLoader(atlas)
    local skeletonDataPtr = spSkeletonData.create(attachmentLoader, skeletonFileName, scale or 1)
    local animations = spSkeletonData.getAnimationList(skeletonDataPtr)
    local ptrToAnimationName = {}
    for name, animation in pairs(animations) do
        ptrToAnimationName[animation[0]] = name
    end
    
    local skeletonData = {
        [0] = skeletonDataPtr,
        animations = animations,
        ptrToAnimationName = ptrToAnimationName,
        events = spSkeletonData.getEventList(skeletonDataPtr),
        bones = spSkeletonData.getBoneList(skeletonDataPtr),
        slots = spSkeletonData.getSlotList(skeletonDataPtr),
        width = spSkeletonData.getWidth(skeletonDataPtr),
        height = spSkeletonData.getHeight(skeletonDataPtr),
        atlas = atlas,
        attachmentLoader = attachmentLoader,
        free = freeSkeletonData,
    }
    return skeletonData
end

local characterDatas = {}
setmetatable(characterDatas, {
    __index = function (s, k)
        -- print("loadCharacterData", k)
        local v = loadSkeletonData("Character", k, 1)
        s[k] = v
        return v
    end,
})
local effectDatas = {}
setmetatable(effectDatas, {
    __index = function (s, k)
        -- print("loadEffectData", k)
        local v = loadSkeletonData("Effect", k, 1)
        s[k] = v
        return v
    end,
})
local function clearSkeletonData(dataList, retainList)
    for name, data in pairs(dataList) do
        local isRetain = retainList and retainList[name]
        if not isRetain then
            data:free()
            dataList[name] = nil
        end
    end
end

function This.loadCharacterData(name)
    return characterDatas[name]
end

function This.loadEffectData(name)
    return effectDatas[name]
end

function This.clearAllEffectData(retainList)
    clearSkeletonData(effectDatas, retainList)
end

function This.clearAllCharacterData(retainList)
    clearSkeletonData(characterDatas, retainList)
end

function This.clearAllEffectData(retainList)
    clearSkeletonData(effectDatas, retainList)
end


function This:createCharacter(name)
    return self:createWithData(characterDatas[name])
end

function This:createEffect(name)
    return self:createWithData(effectDatas[name])
end

function This:createPlayOnceEffect(name, animationName)
    local data = effectDatas[name]
    local effect = self:createWithData(data)
    effect:scheduleUpdate()
    effect.m_playCount = 1
    effect:playAnimation(animationName or "animation", false)
    return effect
end

function This:createLoopPlayEffect(name, animationName)
    local data = effectDatas[name]
    local effect = self:createWithData(data)
    effect:scheduleUpdate()
    effect:playAnimation(animationName or "animation", true)
    return effect
end

local lib_create = lib_skeletonNode.create
function This:createWithData(skeletonData)
    local node = lib_create(skeletonData[0])
    node = registerObject(node, self, {
        m_skeletonData = skeletonData,
        m_timeScale = 1,
        m_rootScale = 1,
        m_ptrToName = skeletonData.ptrToAnimationName,
        m_animations = skeletonData.animations,
        m_boneDatas = skeletonData.bones,
        m_slots = skeletonData.slots,
        m_playCount = -1,
    })
    node:init()
    return node
end

function This:init()
    
end

function This:onExit()
    super.onExit(self)
    self.m_isDead = true
    self.m_listener = nil
end

local lib_getBone = lib_skeletonNode.getBone
function This:getBone(boneName)
    if boneName then
        local boneData = self.m_boneDatas[boneName]
        if boneData then
            return lib_getBone(self[0], boneData.index)
        end
    end
end

local lib_getBoneWorldXY = lib_skeletonNode.getBoneWorldXY
function This:getBoneWorldXY(name)
    local boneData = self.m_boneDatas[name]
    if boneData then
        return lib_getBoneWorldXY(self[0], boneData.index)
    end
end

local lib_getBoneWorldRotation = lib_skeletonNode.getBoneWorldRotation
function This:getBoneWorldRotation(name)
    local boneData = self.m_boneDatas[name]
    if boneData then
        return lib_getBoneWorldRotation(self[0], boneData.index)
    end
end

local lib_getBoneRotation = lib_skeletonNode.getBoneRotation
function This:getBoneRotation(name)
    local boneData = self.m_boneDatas[name]
    if boneData then
        return lib_getBoneRotation(self[0], boneData.index)
    end
end

local lib_setBoneRotation = lib_skeletonNode.setBoneRotation
function This:setBoneRotation(name, rotation)
    local boneData = self.m_boneDatas[name]
    if boneData then
        return lib_setBoneRotation(self[0], boneData.index, rotation)
    end
end

local lib_addBoneRotation = lib_skeletonNode.addBoneRotation
function This:addBoneRotation(name, rotation)
    if rotation == 0 then
        return
    end
    local boneData = self.m_boneDatas[name]
    if boneData then
        lib_addBoneRotation(self[0], boneData.index, rotation)
    end
end

local lib_setBoneScale = lib_skeletonNode.setBoneScale
function This:setBoneScale(name, scaleX, scaleY)
    local boneData = self.m_boneDatas[name]
    if boneData then
        return lib_setBoneScale(self[0], boneData.index, scaleX, scaleY)
    end
end


local lib_setRootParent = lib_skeletonNode.setRootParent
function This:setRootParent(bone)
    lib_setRootParent(self[0], bone)
end

local lib_setRootScale = lib_skeletonNode.setRootScale
function This:setRootScale(scale)
    self.m_rootScale = scale
    lib_setRootScale(self[0], scale)
end

local lib_setRootXY = lib_skeletonNode.setRootXY
function This:setRootXY(x, y)
    lib_setRootXY(self[0], x, y)
end

local lib_setRootRotation = lib_skeletonNode.setRootRotation
function This:setRootRotation(rotation)
    lib_setRootRotation(self[0], rotation)
end

function This:setAngle(angle)
    lib_setRootRotation(self[0], angle)
end

local lib_setSkin = lib_skeletonNode.setSkin
function This:setSkin(skinName)
    lib_setSkin(self[0], skinName)
end

local lib_setMix = lib_skeletonNode.setMix
function This:setMix(fromName, toName, duration)
    local animations = self.m_animations
    local fromAnimation = animations[fromName]
    local toAnimation = animations[toName]
    if not fromAnimation then
        error("setMix failed, fromAnimation is nil, name:", fromName)
        return
    end
    if not toAnimation then
        error("setMix failed, toAnimation is nil, name:", toName)
        return
    end
    lib_setMix(self[0], fromAnimation[0], toAnimation[0], duration)
end


local lib_setAnimation = lib_skeletonNode.setAnimation
function This:setAnimationTrack(trackIndex, animation, isLoop)
    return lib_setAnimation(self[0], trackIndex, animation, isLoop)
end

function This:playAnimation(name, isLoop)
    local animation = self.m_animations[name]
    if not animation then
        print(self._MNAME .. " have not animation name:" .. name)
        return
    end

    self.m_animation = animation
    local track = lib_setAnimation(self[0], 0, animation[0], isLoop)
    return animation, track
end

local lib_setLinkAnimation = lib_skeletonNode.setLinkAnimation
function This:playLinkAnimation(startName, loopName)
    local startAnimation = self.m_animations[startName]
    if not startAnimation then
        print(self._MNAME .. " have not animation name:" .. startName)
        return
    end
    local loopAnimation = self.m_animations[loopName]
    if not loopAnimation then
        print(self._MNAME .. " have not animation name:" .. loopName)
        return
    end

    lib_setLinkAnimation(self[0], startAnimation[0], loopAnimation[0])
    return startAnimation, loopAnimation
end

local lib_addAnimation = lib_skeletonNode.addAnimation
function This:addAnimation(trackIndex, name, isLoop, delay)
    local animation = self.m_animations[name]
    if not animation then
        print(self._MNAME .. " have not animation name:" .. name)
        return
    end
    local track = lib_addAnimation(self[0], trackIndex, animation[0], isLoop, delay or 0)
    return animation, track
end

function This:addAnimationTrack(trackIndex, animation, isLoop, delay)
    return lib_addAnimation(self[0], trackIndex, animation[0], isLoop, delay or 0)
end

local lib_clearTrack = lib_skeletonNode.clearTrack
function This:clearTrack(index)
    lib_clearTrack(self[0], index)
end

local function onAnimationComplete(self, pAnimation, loopCount)
    local name = self.m_ptrToName[pAnimation]
    self.m_loopIndex = self.m_loopIndex + 1
    self.m_loopList[self.m_loopIndex] = name
end

local function onAnimationEvent(self, pAnimation, pEvent)
    local name = self.m_ptrToName[pAnimation]
    local envetName = self.m_skeletonData.events[pEvent].name
    self.m_eventIndex = self.m_eventIndex + 1
    self.m_eventList[self.m_eventIndex] = envetName
    self.m_eventNameList[self.m_eventIndex] = name
end

local lib_updateAnimation = lib_skeletonNode.updateAnimation
function This:updateAnimation(delta)
    delta = delta * self.m_timeScale
    local listener = self.m_listener
    if listener then
        local onComplete = nil
        if listener.onAnimationComplete then
            onComplete = onAnimationComplete
        end
        lib_updateAnimation(self[0], delta, self, onComplete, onAnimationEvent)
    else
        lib_updateAnimation(self[0], delta)
    end
end

local lib_updateTransform = lib_skeletonNode.updateTransform
function This:updateTransform()
    lib_updateTransform(self[0])
end

local lib_getPlayCount = lib_skeletonNode.getPlayCount
local function update(self, delta)
    self:updateAnimation(delta)
    self:updateTransform()
    self:checkListener()
    local playCount = self.m_playCount
    if playCount >= 0 then
        local count, loopCount = lib_getPlayCount(self[0])
        if count >= playCount and loopCount > 0 then
            self:removeFromParent()
        end
    end
end

-- function This:update(delta)
This.update = update

local lib_setFlippedX = lib_skeletonNode.setFlippedX
function This:setFlippedX(isFlippedX)
    lib_setFlippedX(self[0], isFlippedX)
end

local lib_isFlippedX = lib_skeletonNode.isFlippedX
function This:isFlippedX()
    return lib_isFlippedX(self[0])
end

local lib_setIsListenEnabled = lib_skeletonNode.setIsListenEnabled
function This:setAnimationListener(listener)
    self.m_listener = listener
    self.m_eventList = {}
    self.m_loopList = {}
    self.m_loopIndex = 0
    self.m_eventNameList = {}
    self.m_eventIndex = 0
    -- local isListenStart = listener.onAnimationStart ~= nil
    -- local isListenEnd = listener.onAnimationEnd ~= nil
    -- local isListenComplete = listener.onAnimationComplete ~= nil
    -- local isListenEvent = listener.onAnimationEvent ~= nil
    -- lib_setIsListenEnabled(self[0], false, false, isListenComplete, false)
end

local lib_setAttachment = lib_skeletonNode.setAttachment
function This:setAttachment(slotName, attachment)
    local slot = self.m_slots[slotName]
    if slot then
        lib_setAttachment(self[0], slot.index, attachment)
    end
end

local lib_clearAttachment = lib_skeletonNode.clearAttachment
function This:clearAttachment(slotName)
    local slot = self.m_slots[slotName]
    if slot then
        lib_clearAttachment(self[0], slot.index)
    end
end

local lib_getAttachment = lib_skeletonNode.getAttachment
function This:getAttachment(slotName, attachmentName)
    return lib_getAttachment(self[0], self.m_slots[slotName].index, attachmentName)
end

local lib_getSlotAttachment = lib_skeletonNode.getSlotAttachment
function This:getSlotAttachment(slotName)
    return lib_getSlotAttachment(self[0], self.m_slots[slotName].index)
end

local lib_getSlot = lib_skeletonNode.getSlot
function This:getSlot(slotName)
    return lib_getSlot(self[0], self.m_slots[slotName].index)
end

function This:setIsInheritRotation(isInherit)
    lib_skeletonNode.setIsInheritRotation(self[0], isInherit)
end


local lib_resetTrack = lib_skeletonNode.resetTrack
function This:resetTrack(index)
    lib_resetTrack(self[0], index)
end

local lib_getTrack = lib_skeletonNode.getTrack
function This:getTrack(index)
    return lib_getTrack(self[0], index)
end

local lib_getTrackAnimation = lib_skeletonNode.getTrackAnimation
function This:getTrackAnimation(index)
    local aniamtion = lib_getTrackAnimation(self[0], index or 0)
    if aniamtion then
        return self.m_skeletonData.ptrToAnimationName[aniamtion]
    end
end

local lib_getTimeByTrackIndex = lib_skeletonNode.getTimeByTrackIndex
function This:getTimeByTrackIndex(trackIndex)
    return lib_getTimeByTrackIndex(self[0], trackIndex)
end

local lib_setTimeByTrackIndex = lib_skeletonNode.setTimeByTrackIndex
function This:setTimeByTrackIndex(trackIndex, time)
    lib_setTimeByTrackIndex(self[0], trackIndex, time)
end

local lib_setTrackTimeScale = lib_skeletonNode.setTrackTimeScale
function This:setTrackTimeScale(trackIndex, timeScale)
    lib_setTrackTimeScale(self[0], trackIndex, timeScale)
end

function This:setTimeScale(timeScale)
    self.m_timeScale = timeScale
end

function This:getContentSize()
    return self.m_skeletonData.width, self.m_skeletonData.height
end

local lib_setToSetupPose = lib_skeletonNode.setToSetupPose
function This:setToSetupPose()
    lib_setToSetupPose(self[0])
    self:setRootScale(self.m_rootScale)
end

function This:getDrawSlotList(minBoneLength)
    return lib_skeletonNode.getDrawSlotList(self[0], minBoneLength or 0)
end

function This:checkListener()
    local listener = self.m_listener
    if listener then
        local onEvent = listener.onAnimationEvent
        if onEvent then
            for i = 1, self.m_eventIndex do
                onEvent(listener, self.m_eventNameList[i], self.m_eventList[i])
            end
        end
        local onComplete = listener.onAnimationComplete
        if onComplete then
            for i = 1, self.m_loopIndex do
                onComplete(listener, self.m_loopList[i], 0)
            end
        end
        self.m_loopIndex = 0
        self.m_eventIndex = 0
    end
end