local util      = require "xgame.util"
local xruntime   = os.isAndroid and {} or (require "kernel.runtime")

local ANONYMOUS = {}
local dispatching = 0
local suspended = {}
local listeners = {}

xruntime.PermissionStatus = {
    NOT_DETERMINED = 1,
    RESTRICTED = 2,
    DENIED = 3,
    AUTHORIZED = 4,
}

function xruntime.on(event, callback, caller)
    assert(event, 'no event')
    assert(type(callback) == 'function', 'no callback')
    if dispatching > 0 then
        suspended[#suspended + 1] = {xruntime.on, event, callback, caller}
    else
        local map = listeners[event]
        if not map then
            map = {}
            listeners[event] = map
        end
        map[callback] = caller or ANONYMOUS
    end
end

function xruntime.once(event, callback, caller)
    assert(type(callback) == 'function', 'no callback')
    xruntime.on(event, function (...)
        xruntime.off(event, util.callee())
        callback(...)
    end, caller)
end

function xruntime.off(event, callback)
    assert(event, 'no event')
    assert(type(callback) == 'function', 'no callback')
    if dispatching > 0 then
        suspended[#suspended + 1] = {xruntime.off, event, callback}
    else
        local map = listeners[event]
        if map then
            map[callback] = nil
        end
    end
end


local dispatchEvent = function (event, args)
    local map = listeners[event]
    dispatching = dispatching + 1
    if map then
        for callback, caller in pairs(map) do
            if caller == ANONYMOUS then
                callback(args)
            else
                callback(caller, args)
            end
        end
    end
    dispatching = dispatching - 1

    if dispatching == 0 then
        for i, v in ipairs(suspended) do
            v[1](unpack(v, 2))
            suspended[i] = nil
        end
    end
end
if os.isIos then
    xruntime.setDispatcher(dispatchEvent)
    xruntime.setDispatcher = false -- avoid used by others
elseif os.isAndroid then
    xruntime.dispatchEvent = dispatchEvent
end

function xruntime.isMobile()
    return os.isIos or os.isAndroid
end

if os.isAndroid then
    local xluaj = require "xgame.xluaj"
    local AppContext = xluaj.new("kernel/AppContext")
    function xruntime.alert(title, message, ok, no, callback)
        AppContext.alert(title, message, ok, no, function (status)
            callback(status == 'true')
        end)
    end

    function xruntime.installAPK(path)
        AppContext.installAPK(path)
    end

    local function toPermissionStatus(status)
        if status == 'NOT_DETERMINED' then
            return xruntime.PermissionStatus.NOT_DETERMINED
        elseif status == 'RESTRICTED' then
            return xruntime.PermissionStatus.RESTRICTED
        elseif status == 'DENIED' then
            return xruntime.PermissionStatus.DENIED
        else
            return xruntime.PermissionStatus.AUTHORIZED
        end
    end

    function xruntime.getPermissionStatus(permission)
        return toPermissionStatus(AppContext.getPermissionStatus(permission))
    end

    function xruntime.requestPermission(permission, callback)
        AppContext.requestPermission(permission, function (status)
            callback(toPermissionStatus(status))
        end)
    end

    function xruntime.getLanguage()
        return AppContext.getLanguage()
    end
end

return xruntime